#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "gtk/gtk.h"
#include "cron.h"
#include "tcron.h"
#include "fftv.h"

char *to_spc();
char *skip_spc();
char *parse_Cstr(char *ffname, int lno, char *ss, char **end);
void add_tcron_entry(TcronItem *ti);

static void tcron_db_file(char *dbfile)
{
  get_fftv_states_dir(dbfile);
  strcat(dbfile,"/tcron-program.db");
}


void load_program_entries_from_file()
{
  FILE *fp;
  char dbfile[256];

  tcron_db_file(dbfile);

  if ((fp=fopen(dbfile,"r"))==NULL)
    return;

  int lno=0;

  while (!feof(fp)) {
    char tt[256];
    lno++;

    bzero(tt,sizeof(tt));
    fgets(tt,sizeof(tt), fp);
    int len = strlen(tt);

//    dbg("-------- lno:%d %s", lno, tt);

    if (len < 10) {
      continue;
    }

    if (tt[len-1]=='\n')
      tt[len-1]=0;

    TcronItem ti;
    char *ss;

    ti.av_source = skip_spc(tt);
    ss = to_spc(ti.av_source);
    *(ss++)=0;
//    dbg("av_source:%s\n", ti.av_source);

    char *channel=skip_spc(ss);
    ss = to_spc(channel);
    *(ss++)=0;
    ti.channel = atoi(channel);
//    dbg("channel:%d\n", ti.channel);

    ss=skip_spc(ss);
    ti.program = g_strdup(parse_Cstr(dbfile,lno,ss,&ss));
//    dbg("program:%s\n", ti.program);

    ss=skip_spc(ss);
    ti.dayweek = g_strdup(parse_Cstr(dbfile,lno,ss,&ss));
//    dbg("dayweek:%s\n", ti.dayweek);

    ti.month=skip_spc(ss);
    ss = to_spc(ti.month);
    *(ss++)=0;
//    dbg("month:%s\n", ti.month);

    ti.daymonth=skip_spc(ss);
    ss = to_spc(ti.daymonth);
    *(ss++)=0;
//    dbg("daymonth:%s\n", ti.daymonth);

    char *hour=skip_spc(ss);
    ss = to_spc(hour);
    *(ss++)=0;
    ti.hour = atoi(hour);
//    dbg("hour:%d\n",ti.hour);

    char *minute=skip_spc(ss);
    ss = to_spc(minute);
    *(ss++)=0;
    ti.minute = atoi(minute);
//    dbg("minute:%d\n",ti.minute);

    char *duration=skip_spc(ss);
    ss = to_spc(duration);
    *(ss++)=0;
    ti.duration = atoi(duration);
//    dbg("duration:%d\n",ti.duration);

    char *record=skip_spc(ss);
    ss = to_spc(record);
    *(ss++)=0;
    ti.record = atoi(record);
//    dbg("record:%d\n", ti.record);

    add_tcron_entry(&ti);
    g_free(ti.program);
    g_free(ti.dayweek);
  }

  fclose(fp);
}

char *toCstr(char *s);

static char *spc_to_comma(char *str)
{
  char *p = str;
  char tt[512];
  static char uu[512];
  int ttN=0;

  while (*p) {
    char c = *p;
    if (c==' ' || c=='\t')
      c = ',';

    tt[ttN++]=c;
    p++;
  }

  int uuN=0;

  int i;
  for(i=0; i < ttN; i++) {
    if (!(i && tt[i-1] == ',' && tt[i] == ','))
      uu[uuN++] = tt[i];
  }

  uu[uuN]=0;

  return uu;
}


extern GtkWidget *cron_treeview;
extern GArray *cr_entries;

void save_program_entries_to_file()
{
  GtkTreeModel *model = gtk_tree_view_get_model (GTK_TREE_VIEW(cron_treeview));
  GtkTreeIter iter;
  char dbfile[512];


  printf(".. save_program_entries_to_file ..\n");

  if (!gtk_tree_model_get_iter_first(model, &iter))
    return;

  tcron_db_file(dbfile);

  FILE *fp;

  if ((fp=fopen(dbfile, "w"))==NULL) {
    exit(-1);
  }

  FILE *tcron_fp = tcrontab_update_open("fftv");

  do {
    char *av_source;
    gtk_tree_model_get (model, &iter, COLUMN_AV_SOURCE, &av_source, -1);
    gint channel;
    gtk_tree_model_get (model, &iter, COLUMN_CHANNEL, &channel, -1);
    char *program;
    gtk_tree_model_get (model, &iter, COLUMN_PROGRAM, &program, -1);
    char *dayweek;
    gtk_tree_model_get (model, &iter, COLUMN_DAYWEEK, &dayweek, -1);
    char *month;
    gtk_tree_model_get (model, &iter, COLUMN_MONTH, &month, -1);
    char *daymonth;
    gtk_tree_model_get (model, &iter, COLUMN_DAYMONTH, &daymonth, -1);
    int hour;
    gtk_tree_model_get (model, &iter, COLUMN_HOUR, &hour, -1);
    int minute;
    gtk_tree_model_get (model, &iter, COLUMN_MINUTE, &minute, -1);
    int duration;
    gtk_tree_model_get (model, &iter, COLUMN_DURATION, &duration, -1);
    gboolean record;
    gtk_tree_model_get (model, &iter, COLUMN_RECORD, &record, -1);

//    dbg("program:%s\n", program);

    char *cprogram, *cdayweek;
    fprintf(fp,"%s %d %s %s %s %s %d %d %d %d\n",
      av_source, channel, cprogram=toCstr(program), cdayweek=toCstr(dayweek),
        month, daymonth, hour, minute, duration, record);

    char tstr[512];

    sprintf(tstr, "%d %d %s %s %s %s %s %d %s %d ; idle-halt",
      minute, hour, daymonth, month, spc_to_comma(dayweek),
      record ? "ffrecord": "reminder",
      av_source, channel, cprogram, duration);

//    dbg("tstr:%s\n", tstr);

    tcrontab_update_append(tcron_fp, tstr);
    free(cprogram); free(cdayweek);
  } while (gtk_tree_model_iter_next(model, &iter));


  fclose(fp);

  tcrontab_update_close(tcron_fp);
}
