#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>

#include <sys/vfs.h>
#define _I386_STATFS_H   // to skip /usr/include/asm/statfs.h on Mandrake 9.0

#include <linux/videodev.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <sys/time.h>
#include <time.h>
#include <sys/shm.h>
#include <linux/soundcard.h>

#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xresource.h>
#include <X11/extensions/XShm.h>
#include <gtk/gtk.h>

#include <string.h>
#include <ctype.h>


#include "tv-freq.h"
#include "fftv.h"
#include "vidstr.h"
#include "tv-win.h"

#define TV_SOURCE_NAME  "Television"

int vfd;

int min_vxl=48,min_vyl=32,max_vxl,max_vyl;   // min video width / height
static int zero=0,one=1;
struct video_clip *clips;
int clipsN;
extern int chan_idx;
int radio_freq_factor;

char *str_tv_channel;
char *video_source=TV_SOURCE_NAME;

void set_FFTV_MODE_TV()
{
  fftv_mode = FFTV_MODE_TV;
}

void set_FFTV_MODE_RADIO()
{
  fftv_mode = FFTV_MODE_RADIO;
}


void tv_pic_on()
{
    ioctl(vfd, VIDIOCCAPTURE, &one);
}

u_int vidstr_bits;

void set_vidstr(VIDSTR *pvidstr, int xl, int yl)
{
  int idx = pvidstr - vidstr;

  vidstr_bits|=1<<idx;
  pvidstr->xl=xl;  pvidstr->yl=yl;
}


void set_vidstr_xy(VIDSTR *pvidstr, int xl, int yl, int x, int y)
{
  set_vidstr(pvidstr,xl,yl);
  pvidstr->x = x;
  pvidstr->y = y;
}


void clr_vidstr(VIDSTR *pvidstr)
{
  int idx = pvidstr - vidstr;

  vidstr_bits&=~(1<<idx);
  pvidstr->xl=0;
}

void tv_audio_on()
{
  struct video_audio ta;

  bzero(&ta,sizeof(ta));
  ioctl(vfd, VIDIOCGAUDIO, &ta);
  ta.flags &= ~VIDEO_AUDIO_MUTE;
  ioctl(vfd, VIDIOCSAUDIO, &ta);
}


void tv_audio_off()
{
  struct video_audio ta;

  bzero(&ta,sizeof(ta));
  ioctl(vfd, VIDIOCGAUDIO, &ta);
  ta.flags |= VIDEO_AUDIO_MUTE;
  ioctl(vfd, VIDIOCSAUDIO, &ta);
}
void init_v4l_0()
{
   switch (system("v4l-conf")) {
     case -1: /* can't run */
       dbg("could not exec v4l-conf\n");
       break;
     case 0: /* ok */
       break;
     default: /* non-zero return */
       dbg("v4l-conf had some trouble trying to continue anyway\n");
       break;
   }
}

#define FM_PREFIX "FM-"

void set_tuner_type()
{
   if (str_tv_channel && strlen(str_tv_channel) > 2) {
     if (!memcmp(str_tv_channel,FM_PREFIX,strlen(FM_PREFIX))) {
       set_FFTV_MODE_RADIO();
     } else
     if (str_tv_channel && strlen(str_tv_channel)>5)
       video_source=str_tv_channel;
   }
}


void reset_channel_monitor();

static int probe_devices(char *devs[])
{
  int i;

  for(i=0; devs[i]; i++) {
    dbg("trying device %s\n", devs[i]);

    int fd = open(devs[i], O_RDWR);
    if (fd > 0) {
      dbg("fftv will use dev %s\n", devs[i]);
      return fd;
    }
  }

  p_err("could not find any matching device in /dev");
}


void init_v4l()
{
    struct video_picture vpict;
    struct video_capability vcap;
    extern char ffradio_freq[], fftv_freq[];

    sprintf(ffradio_freq,"%s/.ffradio.freqtab",getenv("HOME"));
    sprintf(fftv_freq,"%s/.fftv.freqtab",getenv("HOME"));

    reset_channel_monitor();

    if (fftv_mode & FFTV_MODE_RADIO) {
      static char *radio_devices[]= {
        "/dev/radio",
        "/dev/radio0",
        "/dev/v4l/radio0",
        NULL
      };

      if (vfd)
        close(vfd);

      vfd = probe_devices(radio_devices);
    } else {
      static char *tv_devices[] = {
        "/dev/video",
        "/dev/video0",
        "/dev/video/video0",
        "/dev/v4l/video0",
        NULL
      };

      vfd = probe_devices(tv_devices);
    }

//    dbg("---------- vfd: %d\n",vfd);
    tv_audio_off();

    if (fftv_mode & FFTV_MODE_RADIO) {
      struct video_tuner tuner;

      bzero(&tuner,sizeof(tuner));
      tuner.tuner = 0;
      radio_freq_factor = 16000;

      if (ioctl (vfd, VIDIOCGTUNER, &tuner) < 0)
        radio_freq_factor=16;
      if ((tuner.flags & VIDEO_TUNER_LOW) == 0)
        radio_freq_factor=16;

      init_radio_SYS();

      if (str_tv_channel && strlen(str_tv_channel) > 2) {
        double freq=atof(&str_tv_channel[strlen(FM_PREFIX)]);

        set_tuner_freq(freq * 1000);
      }

      return;
    }

    bzero(&vpict,sizeof(vpict));
    ioctl(vfd,VIDIOCGPICT,&vpict);
#if 1
    vpict.brightness=36000;
    vpict.hue=32768;
    vpict.colour=32112;
    vpict.contrast=22000;
    vpict.whiteness=0;
#endif
    vpict.depth=32;
    vpict.palette=VIDEO_PALETTE_RGB32;
    dbg("%d/%d/%d/%d/%d depth:%d pal:%d\n",
      vpict.brightness,vpict.hue,vpict.colour,vpict.contrast,vpict.whiteness,
      vpict.depth,vpict.palette);
    ioctl(vfd,VIDIOCSPICT,&vpict);

    if (!strcmp(video_source,TV_SOURCE_NAME))
      set_FFTV_MODE_TV();
    else
      set_FFTV_MODE_RADIO();

    setVideoSource(vfd,video_source);
    ioctl(vfd,VIDIOCSYNC,&zero);

    bzero(&vcap,sizeof(vcap));
    ioctl(vfd,VIDIOCGCAP,&vcap);
    max_vxl=vcap.maxwidth;
    max_vyl=vcap.maxheight;
    min_vxl=vcap.minwidth;
    min_vyl=vcap.minheight;
    dbg("min %d:%d   max:%d %d   flag:%x\n",
      min_vxl,min_vyl,max_vxl,max_vyl,vcap.type);
    if (vcap.type & VID_TYPE_CHROMAKEY)
	dbg(">>> VID_TYPE_CHROMAKEY\n");
    if (vcap.type & VID_TYPE_CLIPPING)
        dbg(">>> VID_TYPE_CLIPPING\n");

    if (fftv_mode & FFTV_MODE_TV)
      init_TV_SYS();

//    if (TV_WIN_DISP_MODE_GRAB)
      init_v4l_mbuf();
}

void save_current_channel_name();
void hide_radio_win();
void show_tv_win();
void close_codec_ui_win();

void switch_to_tv()
{
  if (fftv_mode & FFTV_MODE_TV)
    return;
  save_current_channel_name();
  set_FFTV_MODE_TV();
  video_source=TV_SOURCE_NAME;
  set_fftv_cmd_state("Normal");

  hide_radio_win();
  show_tv_win();

  dbg("close vfd %d\n",vfd);
  close(vfd); vfd=0;
  record_status=SWITCH_TV_FM;
  free_chanlist();
  close_codec_ui_win();

  gtk_main_quit();

}

void hide_tv_win();
void show_radio_win();
void fftv_grab_close();

void close_vfd()
{
  close(vfd); vfd=0;
  fftv_grab_close();
}

void switch_to_radio()
{
  dbg("fftv_mode:%d\n",fftv_mode);
  if (fftv_mode & FFTV_MODE_RADIO)
    return;
  save_current_channel_name();
  set_FFTV_MODE_RADIO();
  video_source="FM-RADIO";
  set_fftv_cmd_state("FM-radio");
  str_tv_channel=NULL;

  hide_tv_win();

  show_radio_win();

  dbg("close vfd %d\n",vfd);
  close_vfd();

  record_status=SWITCH_TV_FM;
  free_chanlist();
  close_codec_ui_win();

  gtk_main_quit();
}


static char channel_last_session[]=".last_session";
int find_chanlist_name(char *chname);
void save_channel_name_last_session(char *chname);

void save_current_channel_name()
{
  char *chname=pchan_current->name;
  int rno = pchan_current->rno;
  FILE *fp;
  char fname[256];

  if (find_chanlist_name(chname)==chanlistN)
    return;

  last_session_filename(fname);

  if ((fp=fopen(fname,"w"))==NULL)
    return;

  fprintf(fp,"%d",rno);

  fclose(fp);
}


char *load_channel_last_session()
{
  static char chname[256];
  char *dirname=ffdir_name();
  FILE *fp;
  char fname[256];

  sprintf(fname,"%s/%s",dirname,channel_last_session);

  if ((fp=fopen(fname,"r"))==NULL)
    return NULL;

  chname[0]=0;
  fgets(chname,sizeof(chname),fp);

  fclose(fp);

  return chname;
}

void tv_fm_switch()
{
  if (fftv_mode & FFTV_MODE_TV) {
    switch_to_radio();
  } else {
    switch_to_tv();
  }

  gtk_main_quit();
}


void tv_pic_off()
{
  ioctl(vfd, VIDIOCCAPTURE, &zero);
}


void close_v4l()
{
  if (TV_WIN_DISP_MODE_GRAB)
    fftv_grab_close();

  close(vfd); vfd=0;
}

typedef struct {
 int x,y;
 int xl,yl;
 int count;
} VCLIPS;

VCLIPS *vclipsArr;
int vclipsArrN;

int add_clip(int x, int y, int xl, int yl)
{
   int i;

   for(i=0;i<vclipsArrN;i++)
     if (vclipsArr[i].x==x && vclipsArr[i].y==y &&
         vclipsArr[i].xl==xl && vclipsArr[i].yl==yl) {
       vclipsArr[i].count++;
       return i;
     }

   for(i=0;i<vclipsArrN;i++)
     if (!vclipsArr[i].count)
       break;

   if (i==vclipsArrN) {
     vclipsArr=realloc(vclipsArr,sizeof(VCLIPS)*(vclipsArrN+1));
   }

   vclipsArr[i].x=x;
   vclipsArr[i].y=y;
   vclipsArr[i].xl=xl;
   vclipsArr[i].yl=yl;
   vclipsArr[i].count++;
   return i;
}


void del_clip(int handle)
{
  vclipsArr[handle].count=0;
}


int tv_preview_mode=0;
extern gboolean b_fftv_invisible;

RECT *disp_rect;
int disp_rectN;
void gen_disp_tile();

void set_vwin()
{
    struct video_window vwin;
    int i;
    struct video_clip *cl,*pclip;
    int clN;

    if (tv_preview_mode || b_fftv_invisible)
      return;

    if (tv_win_refresh_mode == TV_WIN_REFRESH_GRAB) {
      if (vidstr_bits) {
        gen_disp_tile();
      } else {
        disp_rectN = 0;
        if (TV_WIN_DISP_MODE_GRAB)
          normal_tv_win_frame_rate();
      }
      return;
    }

    tv_pic_off();
#if 0
    dbg("ofs %d %d %d %d\n", pchan_current->ofs_x0, pchan_current->ofs_y0,
      pchan_current->ofs_x1, pchan_current->ofs_y1);
#endif
    int ofs_x0, ofs_y0;
    int ofs_x1, ofs_y1;
    int vidsizeX;
    int vidsizeY;

    if (win_xl==640 && win_yl==480 && !clipsN) {
      ofs_x0 = pchan_current->ofs_x0;
      ofs_y0 = pchan_current->ofs_y0;
      ofs_x1 = pchan_current->ofs_x1;
      ofs_y1 = pchan_current->ofs_y1;

      vidsizeX = ofs_x0 + ofs_x1 + win_xl;
      vidsizeY = ofs_y0 + ofs_y1 + win_yl;
    } else {
      vidsizeX = win_xl;
      vidsizeY = win_yl;
      ofs_x0 = ofs_y0 = 0;
      ofs_x1 = ofs_y1 = 0;
    }

    vwin.width=MIN(vidsizeX, max_vxl);
    vwin.height=MIN(vidsizeY, max_vyl);

    vwin.x=win_x - ofs_x0;
    vwin.y=win_y - ofs_y0;
    vwin.chromakey=0;
    vwin.flags=0;

    cl=malloc(sizeof(struct video_clip)*(clipsN+vidstrN + 4));
    clN=clipsN;
    memcpy(cl,clips,sizeof(struct video_clip)*clN);

    for(i=0;i<vidstrN;i++) {
      if (vidstr[i].xl) {
        pclip=&cl[clN++];
        pclip->x= ofs_x0 + vidstr[i].x; pclip->y= ofs_y0 + vidstr[i].y;
        pclip->width=vidstr[i].xl; pclip->height=vidstr[i].yl;
        pclip->next=NULL;
      }
    }

    if (win_xl==640 && win_yl==480 && !clipsN) {
        if (ofs_x0) {
          pclip=&cl[clN++];
          pclip->x= 0; pclip->y= 0;
          pclip->width=ofs_x0; pclip->height=vidsizeY;
          pclip->next=NULL;
        }

        if (ofs_x1) {
          pclip=&cl[clN++];
          pclip->x= ofs_x0 + win_xl; pclip->y= 0;
          pclip->width=ofs_x1; pclip->height=vidsizeY;
          pclip->next=NULL;
        }

        if (win_yl >= max_vyl) {
          if (ofs_y0) {
            pclip=&cl[clN++];
            pclip->x= 0; pclip->y= 0;
            pclip->width=win_xl; pclip->height=ofs_y0;
            pclip->next=NULL;
          }

          if (ofs_y1) {
            pclip=&cl[clN++];
            pclip->x= 0; pclip->y= win_yl - ofs_y1;
            pclip->width=win_xl; pclip->height=ofs_y1;
            pclip->next=NULL;
          }
        } else {
        }
    }

    vwin.clips=cl;
    vwin.clipcount=clN;
    ioctl(vfd,VIDIOCSWIN,&vwin);
    free(cl);

    tv_pic_on();
#if	0
    dbg("winxy:  %d %d\n",win_x,win_y);
#endif
}

void grab_draw_local_win(int x, int y, int xl, int yl);

// disp video frame within tv_win
void local_vwin(int wx0,int wy0, int wxl, int wyl)
{
    int wx1=wx0+wxl-1, wy1=wy0+wyl-1;
    int cxl,cyl; // clipping xl,yl;

    if (wx1 >= win_xl)
      wx1=win_xl;
    if (wy1 >= win_yl)
      wy1=win_yl;

    cxl=wx1 - wx0 + 1;    // clipped width height
    cyl=wy1 - wy0 + 1;

    if (cxl > max_vxl)
      cxl=max_vxl;
    if (cyl > max_vyl)
      cyl=max_vyl;


    if (TV_WIN_DISP_MODE_GRAB) {
//      dbg("local_vwin: %d %d %d %d\n", wx0,wy0,wxl,wyl);
      grab_draw_local_win(wx0, wy0, cxl, cyl);
      return;
    }

    struct video_window vwin;

    vwin.width=cxl;
    vwin.height=cyl;

    vwin.x=win_x + wx0;
    vwin.y=win_y + wy0;
    vwin.chromakey=0;
    vwin.flags=0;

    vwin.clips=clips;
    vwin.clipcount=clipsN;

    ioctl(vfd,VIDIOCSWIN,&vwin);
    tv_pic_on();
}



struct video_clip * incr_clips()
{
   clips=realloc(clips,sizeof(struct video_clip)*(clipsN+1));
   return &clips[clipsN++];
}

void clear_clips()
{
  clipsN=0;
}


int mts_mode_active_idx;

#include "tv-audio.h"

SOUND_MODE_M mts_mode_arr[]=
{
 {  VIDEO_SOUND_STEREO, "stereo"  },
 {  VIDEO_SOUND_MONO,   "mono  "  },
 {  VIDEO_SOUND_LANG1,  "lang1 "  },
 {  VIDEO_SOUND_LANG2,  "lang2 "  },
};

#define TV_SOUND_MODE_N (4)
#define FM_SOUND_MODE_N (2)
int mts_mode_arrN=TV_SOUND_MODE_N;

long current_time();
extern void set_audio_channels(int n);

void set_to_mts_mode_active_idx()
{
  struct video_audio vau;

  bzero(&vau,sizeof(struct video_audio));
  ioctl(vfd, VIDIOCGAUDIO, &vau);

  vau.mode=mts_mode_arr[mts_mode_active_idx].mode;

  ioctl(vfd, VIDIOCSAUDIO, &vau);
  save_channel_state("mts",mts_mode_active_idx);

  dbg("set_mts_mode_active_idx:%d %s\n",mts_mode_active_idx,mts_mode_arr[mts_mode_active_idx].name);
}

int get_audio_channel_no()
{
  if (mts_mode_arr[mts_mode_active_idx].mode == VIDEO_SOUND_STEREO)
    return 2;
  else
    return 1;
}

void switch_sound_mode()
{
  static time_t last_time=0;

  if (last_time && current_time() - last_time < 3) {
    if (fftv_mode & FFTV_MODE_RADIO)
      mts_mode_active_idx=(mts_mode_active_idx+1)% FM_SOUND_MODE_N;
    else
      mts_mode_active_idx=(mts_mode_active_idx+1)% TV_SOUND_MODE_N;

    set_to_mts_mode_active_idx();
  }

  last_time=current_time();

  dbg("mts_mode_active_idx:%d %s\n",mts_mode_active_idx,mts_mode_arr[mts_mode_active_idx].name);

  if (fftv_mode & FFTV_MODE_TV) {
    draw_vol_status(mts_mode_arr[mts_mode_active_idx].name,1);
  } else {
  }
}

#define ADJ_STEP (655)
#define MAX_VAL (65535)

typedef struct {
  char *name;
  int offset;
} PICT_ADJ;

#define VP_OFFSET(field) ( (int) (&((struct video_picture *)0)->field)  )

PICT_ADJ pic_adj[]=
{
  {"brightness",VP_OFFSET(brightness)},
  {"contrast",	VP_OFFSET(contrast)},
  {"color",    VP_OFFSET(colour)},
  {"hue",	VP_OFFSET(hue)},
};

static int pic_adjN=sizeof(pic_adj)/sizeof(pic_adj[0]);
static int pic_adj_idx;
static struct video_picture vpict;

static __u16 *val_ptr(int idx)
{
  PICT_ADJ *pa=&pic_adj[idx];
  int ofs=pa->offset;

  return (__u16  *)(((char *)&vpict) + ofs);
}

void set_pic_adj_idx(char *name)
{
  int i;

  for(i=0;i<4;i++) {
    if (!strcmp(pic_adj[i].name, name)) {
      pic_adj_idx = i;
      return;
    }
  }
}

void draw_pict_adj_status(char *imgfname, int val);

static void disp_current_val()
{
  PICT_ADJ *pa=&pic_adj[pic_adj_idx];
  __u16  *pval=val_ptr(pic_adj_idx);

  ioctl(vfd,VIDIOCGPICT,&vpict);

  draw_pict_adj_status(pa->name,*pval*100/MAX_VAL);
}

extern int pic_adj_menu_mode;
static char *original_cmd_state;
static char *get_fftv_cmd_state();

void pic_adj_select_menu()
{
  if (!strcmp(get_fftv_cmd_state(),"Menu"))
    return;
  original_cmd_state = get_fftv_cmd_state();
  clear_remkeys();
  disp_current_val();
  set_fftv_cmd_state("Menu");
  pic_adj_menu_mode=1;
}

void stop_thread_wait_clear_pict_select();

void pic_adj_select_menu_off()
{
  dbg("menu off\n");
  if (ppicadj_str->xl) {
    ppicadj_str->xl=0;
    set_vwin();
    gtk_widget_hide(ppicadj_str->button);
  }
  ppicadj_str->clear_time=0;
  pic_adj_menu_mode=0;
  dbg("menu off %s\n", original_cmd_state);
  set_fftv_cmd_state(original_cmd_state);
}


void pic_adj_select_up()
{
  pic_adj_idx=(pic_adj_idx+1) % pic_adjN;
  disp_current_val();
}

void pic_adj_select_down()
{
  pic_adj_idx--;
  if (pic_adj_idx<0)
    pic_adj_idx=pic_adjN-1;
  disp_current_val();
}


void pic_val_inc()
{
  __u16  *pval=val_ptr(pic_adj_idx);
  PICT_ADJ *pa=&pic_adj[pic_adj_idx];

  ioctl(vfd,VIDIOCGPICT,&vpict);

  if (*pval+ADJ_STEP < MAX_VAL)
    *pval+=ADJ_STEP;
  else
    *pval=MAX_VAL;

  ioctl(vfd,VIDIOCSPICT,&vpict);

  save_channel_state(pa->name,*pval);

  disp_current_val();
}


void pic_val_dec()
{
  __u16  *pval=val_ptr(pic_adj_idx);
  PICT_ADJ *pa=&pic_adj[pic_adj_idx];

  ioctl(vfd,VIDIOCGPICT,&vpict);

  if (*pval-ADJ_STEP > 0)
    *pval-=ADJ_STEP;
  else
    *pval=0;

  ioctl(vfd,VIDIOCSPICT,&vpict);

  save_channel_state(pa->name,*pval);

  disp_current_val();
}

void get_fftv_states_dir(char *fftv_states_dir)
{
  sprintf(fftv_states_dir,"%s/.fftv.states",getenv("HOME"));
}

static char *channel_state_file_name(char *state_name, int b_mkdir)
{
  char *chname=chanlist[chan_idx].name;
  char chan_dir[256],chan_dayweek_dir[256],chan_dayweek_hour_dir[256];
  char fftv_states_dir[256];
  struct stat st;
  static char fullname[256];
  struct tm ltm;
  time_t ltime=time(NULL);
  localtime_r(&ltime,&ltm);
#if 0
  if (!chanlist)
    p_err("chanlist is null");
  if (chan_idx < 0 || chan_idx>=chanlistN)
    p_err("bad chan_idx");
#endif
  chname=chanlist[chan_idx].name;

  get_fftv_states_dir(fftv_states_dir);

  sprintf(chan_dir,"%s/%s",fftv_states_dir,chname);
  sprintf(chan_dayweek_dir,"%s/%d",chan_dir,ltm.tm_wday);
  sprintf(chan_dayweek_hour_dir,"%s/%d",chan_dayweek_dir,ltm.tm_hour);

  if (stat(chan_dayweek_hour_dir,&st)) {
    if (stat(chan_dayweek_dir,&st)) {
      if (stat(chan_dir,&st)) {
         if (stat(fftv_states_dir,&st)) {
           if (b_mkdir && mkdir(fftv_states_dir,0755)) {
             perror(fftv_states_dir);
             return NULL;
           }
         }

         if (b_mkdir && mkdir(chan_dir,0755)) {
           perror(chan_dir);
           return NULL;
         }
      }

      if (b_mkdir && mkdir(chan_dayweek_dir,0755)) {
        perror(chan_dayweek_dir);
        return NULL;
      }
    }

    if (b_mkdir && mkdir(chan_dayweek_hour_dir,0755)) {
      perror(chan_dayweek_hour_dir);
      return NULL;
    }
  }

  sprintf(fullname,"%s/%s",chan_dayweek_hour_dir,state_name);
  return fullname;
}

void save_channel_state(char *st_name, int val)
{
  FILE *fp;

  char *fullname=channel_state_file_name(st_name ,1);

  if (!fullname)
    return;

  if ((fp=fopen(fullname,"w"))==NULL) {
    error("cannot not create file %s\n",fullname);
    return;
  }

//  dbg("fullname:%s\n",fullname);

  fprintf(fp,"%d",val);
  fclose(fp);
}


int get_channel_state(char *st_name)
{
  FILE *fp;
  int val=0;

  char *fullname=channel_state_file_name(st_name,0);

  if (!fullname)
    return -1;

  if ((fp=fopen(fullname,"r"))==NULL) {
    return -1;
  }

  fscanf(fp,"%d",&val);
  fclose(fp);

  return val;
}

void set_soundcard_volume(int vol);

void restore_channel_states()
{
  int i;
  int val;

  ioctl(vfd,VIDIOCGPICT,&vpict);

  for(i=0;i<pic_adjN;i++) {
    char *st_name=pic_adj[i].name;
    __u16  *pval=val_ptr(i);
    val=get_channel_state(st_name);

    if (val>=0) {
      *pval=val;
//      dbg("%s  %d\n",st_name,val);
    }
  }

  ioctl(vfd,VIDIOCSPICT,&vpict);

  val=get_channel_state("mts");

  if (val>=0) {
    struct video_audio vau;
    bzero(&vau,sizeof(struct video_audio));

    ioctl(vfd, VIDIOCGAUDIO, &vau);

    mts_mode_active_idx=val;
    vau.mode=mts_mode_arr[val].mode;

    ioctl(vfd, VIDIOCSAUDIO, &vau);
//    dbg("mts:%s\n",mts_mode_arr[val].name);
  }

  val=get_channel_state("volume");

  // don't restore volume if RECORD_ON (batch mode -t option)
  if (val>=0 && record_status!=RECORD_ON) {
    set_soundcard_volume(val);
  }

}


int set_xkey_cmd_state(char *state_name);

char *fftv_cmd_state_str;

void set_fftv_cmd_state(char *state)
{
  fftv_cmd_state_str=state;
//  dbg("set_fftv_cmd_state:%s\n", state);
#if LIRC
  set_lirc_cmd_state(state);
#endif
  set_xkey_cmd_state(state);
}

static char *get_fftv_cmd_state()
{
  return fftv_cmd_state_str;
}


time_t start_rec_time, accu_rec_time;

void set_record_status(int status)
{
  if (status==RECORD_STOP) {
    start_rec_time=0;
    accu_rec_time=0;
  } else
  if (status==RECORD_PAUSE) {
    accu_rec_time += current_time() - start_rec_time;
  } else
  if (status==RECORD_ON) {
    start_rec_time = current_time();
  }

  record_status = status;
}


char output_filename[128];

char *print_time(char *tstr,float secs)
{
  if (secs < 120.0)
    sprintf(tstr,"%.0fs",secs);
  else
    sprintf(tstr,"%.1fm",secs / 60.0);

  return tstr;
}

void draw_rec_output_status(char *s);

void disp_rec_output_status()
{
  struct stat sta;
  struct statfs stafs;
  char str[32];
  float total_rec_time = accu_rec_time + current_time() - start_rec_time;
  float bytes_rate;
  char a[16], b[16];

  stat(&output_filename[0],&sta);
  statfs(".",&stafs);

  if (total_rec_time)
    bytes_rate =(float)sta.st_size / total_rec_time;
  else
    bytes_rate = 1;
   print_time(a,total_rec_time),

  sprintf(str,"%s %s/%s",output_filename,
   print_time(a,total_rec_time),
   print_time(b,(float)stafs.f_bavail * stafs.f_bsize / bytes_rate));
  draw_rec_output_status(str);
}


extern int xwin;

int fftv_mode_prev;

void init_fftv()
{
    struct video_audio audio;

    if ((fftv_mode & FFTV_MODE_TV) && TV_WIN_DISP_MODE_GRAB) {
      tv_win_refresh_mode = TV_WIN_REFRESH_GRAB;
      create_x_shm_image();
    } else
      tv_win_refresh_mode = TV_WIN_REFRESH_OVERLAY;

    if (fftv_mode_prev == fftv_mode && vfd)
      return;

    fftv_mode_prev = fftv_mode;

    set_tuner_type();

    if (xwin && (fftv_mode & FFTV_MODE_TV))
      init_v4l_0();

    if (xwin) {
      init_v4l();
    } else {
      if (fftv_mode & FFTV_MODE_TV)
        setVideoSource(0,video_source);
    }

    if ((fftv_mode & FFTV_MODE_TV) && str_tv_channel) {
      set_channel_by_name(str_tv_channel);
    }

    /* unmute audio */
    ioctl(vfd, VIDIOCGAUDIO, &audio);
    audio.flags &= ~VIDEO_AUDIO_MUTE;
    ioctl(vfd, VIDIOCSAUDIO, &audio);
}
