#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include "gtk/gtk.h"
#include "tv-freq.h"
#include "fftv.h"
#include <gdk/gdkkeysyms.h>


static GtkWidget *radio_list_window = NULL;
static GtkWidget *vbox;
static GtkWidget *hbox;
static GtkWidget *sw;
static GtkWidget *treeview;
static GtkWidget *button;
static GtkTreeModel *model;
static GtkWidget *menubar;


typedef struct {
  gchar *name;
  gint  number;
  gchar *frequency;
  gboolean editable;
} Item;

enum {
  COLUMN_NAME,
  COLUMN_NUMBER,
  COLUMN_FREQ,
  COLUMN_EDITABLE,
  NUM_COLUMNS
};

static GArray *stations = NULL;

static char *freq_str(float freq)
{
  static char tt[16];

  sprintf(tt,"%.2f",freq);

  int len=strlen(tt);
  if (tt[len-1]=='0')
    tt[len-1]=0;

  return tt;
}


static int qcmp_freq(const void *aa, const void *bb)
{
  Item *a=(Item *)aa, *b=(Item *)bb;
  float fa=atof(a->frequency), fb=atof(b->frequency);

  if (fa < fb)
    return 1;
  if (fa > fb)
    return -1;

  return 0;
}


static void radio_add_items ()
{
  g_return_if_fail (stations != NULL);
  int i;

  for(i=0;i<chanlistN;i++) {
    Item foo;
    char tt[16];

    foo.name = g_strdup(chanlist[i].name);
    foo.number = chanlist[i].rno;
    sprintf(tt,"%.2f",(float)chanlist[i].freq/1000);
    foo.frequency = g_strdup(freq_str((float)chanlist[i].freq/1000));
    foo.editable = TRUE;
    g_array_append_vals (stations, &foo, 1);
  }

  g_array_sort (stations,qcmp_freq);
}

static GtkTreeModel *radio_create_model (void)
{
  gint i = 0;
  GtkListStore *model;
  GtkTreeIter iter;

  /* create array */
  stations = g_array_sized_new (FALSE, FALSE, sizeof (Item), 1);

  radio_add_items ();

  /* create list store */
  model = gtk_list_store_new (NUM_COLUMNS,G_TYPE_STRING, G_TYPE_INT,
                              G_TYPE_STRING,G_TYPE_BOOLEAN);

  /* add items */
  for (i = 0; i < stations->len; i++) {
      gtk_list_store_append (model, &iter);

      gtk_list_store_set (model, &iter,
                          COLUMN_NAME,
                          g_array_index (stations, Item, i).name,
                          COLUMN_NUMBER,
                          g_array_index (stations, Item, i).number,
                          COLUMN_FREQ,
                          g_array_index (stations, Item, i).frequency,
                          COLUMN_EDITABLE,
                          g_array_index (stations, Item, i).editable,
                          -1);
  }

  return GTK_TREE_MODEL (model);
}


void add_item_to_list (char *name, int rno, int freqkhz)
{
  Item foo;
  GtkTreeIter iter;
  GtkTreeModel *model = gtk_tree_view_get_model(GTK_TREE_VIEW(treeview));

  g_return_if_fail (stations != NULL);

  foo.name = g_strdup(name);
  foo.number = rno;
  foo.frequency = g_strdup(freq_str((float)freqkhz/1000));
  foo.editable = TRUE;
  g_array_append_vals (stations, &foo, 1);

  gtk_list_store_append (GTK_LIST_STORE (model), &iter);
  gtk_list_store_set (GTK_LIST_STORE (model), &iter,
                      COLUMN_NAME, foo.name,
                      COLUMN_NUMBER, foo.number,
                      COLUMN_FREQ, foo.frequency,
                      COLUMN_EDITABLE, foo.editable,
                      -1);
}

int get_max_rno();
void add_new_station(int rno, int freqkhz);

static void radio_add_item (GtkWidget *button, gpointer data)
{
  int rno=get_max_rno() + 1;

  add_new_station(rno, 88000);
  add_item_to_list(chanlist[chanlistN-1].name,rno,88000);
}


void delete_current_channel();
void radio_set_selection_by_name(gchar *name);

static void
remove_item (GtkWidget *widget, gpointer data)
{
  GtkTreeIter iter;
  GtkTreeSelection *selection=gtk_tree_view_get_selection(GTK_TREE_VIEW(treeview));

  if (!gtk_tree_selection_get_selected (selection, NULL, &iter))
    return;

  gint i;
  GtkTreePath *path;

  path = gtk_tree_model_get_path (model, &iter);
  i = gtk_tree_path_get_indices (path)[0];
  gtk_list_store_remove (GTK_LIST_STORE (model), &iter);

  delete_current_channel();

  g_array_remove_index (stations, i);

  if (i==chanlistN)
    i = chanlistN -1;
#if 1
  if (i>=0)
    radio_set_selection_by_name(g_array_index(stations,Item,i).name);
#endif
  gtk_tree_path_free (path);
}

int rename_station(int rno, const gchar *new);
static void set_station_freq(char *name, float freq);
CHANLIST *rno2pcha(int rno);
void delete_rno_file(int rno);
int save_channel(CHANLIST *pcha);

static int set_station_rno(int old_rno, int rno)
{
  CHANLIST *pchan=rno2pcha(old_rno);

  if (!rno || rno2pcha(rno)) {
    dbg("station exits %d\n",rno);
    return -1;
  }

  delete_rno_file(old_rno);

  pchan->rno = rno;

  save_channel(pchan);

  return 0;
}


static void
cell_edited (GtkCellRendererText *cell,
             const gchar         *path_string,
             const gchar         *new_text,
             gpointer             data)
{
  GtkTreeModel *model = (GtkTreeModel *)data;
  GtkTreePath *path = gtk_tree_path_new_from_string (path_string);
  GtkTreeIter iter;

  gint *column;

  column = g_object_get_data (G_OBJECT (cell), "column");

  gtk_tree_model_get_iter (model, &iter, path);

  int i = gtk_tree_path_get_indices (path)[0];
  int rno =  g_array_index (stations, Item, i).number;
  gchar *old_text;

  gtk_tree_model_get (model, &iter, column, &old_text, -1);

  switch (GPOINTER_TO_INT (column)) {
    case COLUMN_NAME:
      {
        if (rename_station(rno,new_text))
          break;

        g_free (old_text);

        g_array_index (stations, Item, i).name = g_strdup (new_text);

        gtk_list_store_set (GTK_LIST_STORE (model), &iter, column,
                            g_array_index (stations, Item, i).name, -1);
      }
      break;
    case COLUMN_NUMBER:
      {
        int newval=atoi(new_text);

        if (set_station_rno(rno, newval))
          break;

        g_array_index (stations, Item, i).number = newval = atoi (new_text);

        gtk_list_store_set (GTK_LIST_STORE (model), &iter, column,
                            g_array_index (stations, Item, i).number, -1);
      }
      break;
    case COLUMN_FREQ:
      {
        gchar *old_text=NULL;
        float newval;

        newval=atof(new_text);
        if (newval < 80.0 || newval>109.0)
          newval = atof(old_text);

        g_free (old_text);

        g_array_index(stations, Item, i).frequency = g_strdup(freq_str(newval));

        gtk_list_store_set (GTK_LIST_STORE (model), &iter, column,
                            g_array_index (stations, Item, i).frequency, -1);
        set_station_freq(g_array_index(stations, Item, i).name, newval);
      }
      break;
  }

  gtk_tree_path_free (path);
}

static void radio_add_columns (GtkTreeView *treeview)
{
  GtkCellRenderer *renderer;
  GtkTreeModel *model = gtk_tree_view_get_model (treeview);

  /* name column */

  renderer = gtk_cell_renderer_text_new ();
  g_signal_connect (G_OBJECT (renderer), "edited",
                    G_CALLBACK (cell_edited), model);
  g_object_set_data (G_OBJECT (renderer), "column", (gint *)COLUMN_NAME);

  gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (treeview),
                                               -1, "Station Name", renderer,
                                               "text", COLUMN_NAME,
                                               "editable", COLUMN_EDITABLE,
                                               NULL);

  /* number column */
  renderer = gtk_cell_renderer_text_new ();
  g_signal_connect (G_OBJECT (renderer), "edited",
                    G_CALLBACK (cell_edited), model);
  g_object_set_data (G_OBJECT (renderer), "column", (gint *)COLUMN_NUMBER);


  gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (treeview),
                                               -1, "Access #", renderer,
                                               "text", COLUMN_NUMBER,
                                               "editable", COLUMN_EDITABLE,
                                               NULL);

  /* frequency column */

  renderer = gtk_cell_renderer_text_new ();
  g_signal_connect (G_OBJECT (renderer), "edited",
                    G_CALLBACK (cell_edited), model);
  g_object_set_data (G_OBJECT (renderer), "column", (gint *)COLUMN_FREQ);

  gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (treeview),
                                               -1, "MHz", renderer,
                                               "text", COLUMN_FREQ,
                                               "editable", COLUMN_EDITABLE,
                                               NULL);
}

static void callback_row_selected(GtkTreeSelection *treeselection,
                                  gpointer user_data)
{
  GtkTreeIter iter;

  if (!gtk_tree_selection_get_selected (treeselection, NULL, &iter))
    return;

  gint i;
  GtkTreePath *path;
  gchar *name;

  path = gtk_tree_model_get_path (model, &iter);
  i = gtk_tree_path_get_indices (path)[0];
  gtk_tree_path_free(path);

  name=g_array_index (stations, Item, i).name;

  printf("selected %s\n",name);
  set_channel_by_name(name);
}

void do_editable_cells ();

void show_radio_list_window()
{
  if (radio_list_window) {
    gtk_widget_show(radio_list_window);
    gtk_window_present(GTK_WINDOW(radio_list_window));
  }
  else
    do_editable_cells();

  if (pchan_current)
    radio_set_selection_by_name(pchan_current->name);
}

void hide_radio_list_window()
{
  if (radio_list_window)
    gtk_widget_hide(radio_list_window);
}

static void callback_win_delete()
{
  hide_radio_list_window();
}

void radio_set_selection_by_name(gchar *name)
{
  if (!treeview)
    return;

  GtkTreeSelection *selection=gtk_tree_view_get_selection(GTK_TREE_VIEW(treeview));
  GtkTreeModel *model = gtk_tree_view_get_model(GTK_TREE_VIEW(treeview));
  GtkTreeIter iter;
  gboolean found=FALSE;

  if (!gtk_tree_model_get_iter_first(model, &iter))
    return;

  do {
    gchar *tname;

    gtk_tree_model_get(model,&iter,COLUMN_NAME, &tname,-1);

    if (!strcmp(tname,name)) {
      found=TRUE;
      break;
    }
  } while (gtk_tree_model_iter_next(model, &iter));

  if (found)
    gtk_tree_selection_select_iter(selection,&iter);

  gtk_tree_view_scroll_to_cell(GTK_TREE_VIEW(treeview),
    gtk_tree_model_get_path(model,&iter),NULL,FALSE,0.5,0.0);
}

static void radio_rescan( GtkWidget *w,
                         gpointer   data )
{
  radio_scan();
}

static GtkItemFactoryEntry menu_items[] = {
  { "/_File",         NULL,         NULL, 0, "<Branch>" },
  { "/File/Rescan",   NULL, radio_rescan, 0, NULL },
  { "/File/sep1",     NULL,         NULL, 0, "<Separator>" },
  { "/File/Close",     "<control>X", callback_win_delete, 0, NULL },
};


static void get_main_menu( GtkWidget  *window)
{
  GtkItemFactory *item_factory;
  GtkAccelGroup *accel_group;
  gint nmenu_items = sizeof (menu_items) / sizeof (menu_items[0]);

  accel_group = gtk_accel_group_new ();

  item_factory = gtk_item_factory_new (GTK_TYPE_MENU_BAR, "<main>",
				       accel_group);

  gtk_item_factory_create_items (item_factory, nmenu_items, menu_items, NULL);

  gtk_window_add_accel_group (GTK_WINDOW (window), accel_group);

  menubar = gtk_item_factory_get_widget (item_factory, "<main>");
}


static gboolean callback_key_press(GtkWidget *widget, GdkEventKey *event, gpointer user_data)
{
  if (event->keyval == GDK_Delete) {
    remove_item(NULL,NULL);
  } else {
    gchar chr=0;
    if (event->string)
      chr=event->string[0];
    feed_xkey(event->keyval,event->state,chr);
  }

  return TRUE;
}


void do_editable_cells ()
{

  /* create radio_list_window, etc */
  radio_list_window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_window_set_wmclass(GTK_WINDOW(radio_list_window),"ffradio","ffradio");

  GtkWidget *win_icon=gtk_image_new_from_file(ICON_BASENAME "/fftv.png");
  gtk_window_set_icon(GTK_WINDOW(radio_list_window),
                      gtk_image_get_pixbuf(GTK_IMAGE(win_icon)));


  gtk_window_set_title (GTK_WINDOW (radio_list_window), "FM Radio Stations");
  gtk_container_set_border_width (GTK_CONTAINER (radio_list_window), 5);

  g_signal_connect (G_OBJECT (radio_list_window), "delete_event",
                      G_CALLBACK (callback_win_delete), NULL);

  vbox = gtk_vbox_new (FALSE, 5);

  gtk_container_add (GTK_CONTAINER (radio_list_window), vbox);

  get_main_menu(radio_list_window);

  gtk_box_pack_start (GTK_BOX (vbox), menubar, FALSE, FALSE, 0);

  gtk_box_pack_start (GTK_BOX (vbox),
                      gtk_label_new ("List of FM Radio Station"),
                      FALSE, FALSE, 0);

  sw = gtk_scrolled_window_new (NULL, NULL);
  gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (sw),
                                       GTK_SHADOW_ETCHED_IN);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (sw),
                                  GTK_POLICY_AUTOMATIC,
                                  GTK_POLICY_AUTOMATIC);
  gtk_box_pack_start (GTK_BOX (vbox), sw, TRUE, TRUE, 0);

  /* create model */
  model = radio_create_model ();

  /* create tree view */
  treeview = gtk_tree_view_new_with_model (model);
  g_object_unref (G_OBJECT (model));
  gtk_tree_view_set_rules_hint (GTK_TREE_VIEW (treeview), TRUE);

  PangoFontDescription *font=pango_context_get_font_description
                             (gtk_widget_get_pango_context (treeview));
  pango_font_description_set_size(font,PANGO_SCALE*16);
  gtk_widget_modify_font(treeview, font);


  GtkTreeSelection *tree_selection =
     gtk_tree_view_get_selection (GTK_TREE_VIEW (treeview));
  gtk_tree_selection_set_mode (tree_selection, GTK_SELECTION_SINGLE);

  radio_add_columns (GTK_TREE_VIEW (treeview));

  gtk_container_add (GTK_CONTAINER (sw), treeview);


  /* some buttons */
  hbox = gtk_hbox_new (TRUE, 4);
  gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);

  button = gtk_button_new_with_label ("Add station");
  g_signal_connect (G_OBJECT (button), "clicked",
                    G_CALLBACK (radio_add_item), model);
  gtk_box_pack_start (GTK_BOX (hbox), button, TRUE, TRUE, 0);

  button = gtk_button_new_with_label ("Remove station");
  g_signal_connect (G_OBJECT (button), "clicked",
                    G_CALLBACK (remove_item), treeview);
  gtk_box_pack_start (GTK_BOX (hbox), button, TRUE, TRUE, 0);

  gtk_window_set_default_size (GTK_WINDOW (radio_list_window), 300, 300);

  g_signal_connect (G_OBJECT (radio_list_window), "delete_event",
                    G_CALLBACK (gtk_main_quit), NULL);

  gtk_widget_show_all (radio_list_window);


  g_signal_connect (G_OBJECT(tree_selection), "changed",
                    G_CALLBACK (callback_row_selected), NULL);

  g_signal_connect(G_OBJECT(treeview),"key-press-event",
     G_CALLBACK(callback_key_press), NULL);
}

CHANLIST *find_channel_name(char *name)
{
  int i;

  for(i=0;i<chanlistN;i++) {
    if (strcmp(chanlist[i].name,name))
        continue;

    return &chanlist[i];
  }

  return NULL;
}

extern char *ffdir_name();
CHANLIST *rno2pcha(int rno);

int rename_station(int rno, const gchar *new)
{
  int rstatus=0;

  CHANLIST *pcha= rno2pcha(rno);

  if (!pcha)
    return -1;

  free(pcha->name);
  pcha->name = strdup(new);
  pcha->flag = 0;

  rstatus = save_channel(pcha);

  return rstatus;
}


void set_freq_disp(CHANLIST *pcha);
void draw_radio_freq(float freq);

static void set_station_freq(char *name, float freq)
{
  CHANLIST *pchan=find_channel_name(name);

  pchan->freq = freq * 1000;

  draw_radio_freq(freq);
  set_freq_disp(pchan);
  save_channel(pchan);
}

