#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <linux/videodev.h>
#include <gtk/gtk.h>
#include <string.h>
#include "tv-freq.h"
#include "fftv.h"

static GtkWidget *scan_window, *button_stop, *vbox, *label_freq, *hsepa;
static GtkWidget *prog_bar;

static gboolean b_stopped;

void callback_button_stop(GtkButton *button, gpointer user_data)
{
  b_stopped = TRUE;
  printf("stop\n");
}

extern int radio_freq_factor, vfd;

int freq_khz_radiodev(int freq)
{
  return freq*radio_freq_factor/1000;
}


// input is KHz
static int radio_setfreq(int freq)
{
    int ifreq = freq_khz_radiodev(freq);
    return ioctl(vfd, VIDIOCSFREQ, &ifreq);
}


static int
radio_getsignal()
{
    struct video_tuner vt;
    int signal;

    bzero(&vt,sizeof(vt));
    ioctl (vfd, VIDIOCGTUNER, &vt);
    signal=vt.signal>>13;

    return signal;
}

typedef struct {
   int chidx;
   int s;  // strength
} FM_SLOT;

static int qmp_strength(const void *aa, const void *bb)
{
  FM_SLOT *a=(FM_SLOT *)aa,*b=(FM_SLOT *)bb;

  return a->s - b->s;
}

#define FM_SLOT_N (411)

#define CHIDX_KHZ(i) (87500 + (i)*50)

static void create_scan_window()
{
  scan_window = gtk_window_new (GTK_WINDOW_TOPLEVEL);

  gtk_container_set_border_width (GTK_CONTAINER (scan_window), 5);

  vbox = gtk_vbox_new (FALSE, 5);

  label_freq = gtk_label_new("100.12 MHz");
  gtk_box_pack_start (GTK_BOX (vbox), label_freq, TRUE, TRUE, 0);

  hsepa = gtk_hseparator_new();
  gtk_box_pack_start (GTK_BOX (vbox), hsepa, TRUE, TRUE, 0);

  prog_bar = gtk_progress_bar_new ();
  gtk_box_pack_start (GTK_BOX (vbox), prog_bar, TRUE, TRUE, 0);

  button_stop = gtk_button_new_with_label ("Stop");
  gtk_box_pack_start (GTK_BOX (vbox), button_stop, TRUE, TRUE, 0);
  g_signal_connect (G_OBJECT(button_stop),"clicked",
      G_CALLBACK(callback_button_stop),NULL);

  gtk_container_add (GTK_CONTAINER(scan_window), vbox);

  gtk_widget_show_all(scan_window);
}

int get_max_rno()
{
  int mrno=0, i;

  for(i=0;i<chanlistN;i++) {
    if (mrno < chanlist[i].rno)
      mrno = chanlist[i].rno;
  }

  return mrno;
}

extern char ffradio_freq[];

void add_new_station(int rno, int freqkhz)
{
  char chname[16];
  char ffradio_freq_name[256];
  FILE *fp;

  sprintf(chname,"FM%d",rno);
  sprintf(ffradio_freq_name,"%s/%d",ffradio_freq,rno);

  if ((fp=fopen(ffradio_freq_name,"w"))==NULL)
    p_err("cannot create %s",ffradio_freq_name);

  fprintf(fp,"%d 0\n%s",freqkhz,chname);
  fclose(fp);

  chanlist=realloc(chanlist,sizeof(CHANLIST)*(chanlistN+1));
  chanlist[chanlistN].name=strdup(chname);
  chanlist[chanlistN].rno=rno;
  chanlist[chanlistN].freq=freqkhz;
  chanlistN++;

  pchan_current=NULL;
}

void add_item_to_list (char *name, int rno, int freqkhz);

void radio_scan()
{
  int i;
  FM_SLOT fm_slot[FM_SLOT_N], o_fm_slot[FM_SLOT_N];
  int cutoff;
  int maxidx0,maxidx1,cnt,flat=0;
  struct stat st;

  create_scan_window();

  tv_audio_off();

//  dbg("scan radio ...\n");
  int last_percent=-1;

  for(i=0;i<FM_SLOT_N;i++) {
      int freq = CHIDX_KHZ(i);
      int j;
      int s=0;
      char tt[32];

      double frac=(double)i/FM_SLOT_N;
      int percent=(int)(frac*100) + 1;

      if (percent != last_percent) {
        last_percent = percent;

        sprintf(tt,"%.2f MHz",(float)freq/1000);
        gtk_label_set_text(GTK_LABEL(label_freq),tt);

        sprintf(tt,"%d%%",percent);
        gtk_progress_bar_set_text(GTK_PROGRESS_BAR(prog_bar),tt);
        gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(prog_bar),frac);
      }

      process_gtk_events();

      if (b_stopped)
        goto ret;

      radio_setfreq(freq);
      usleep(10000);
#define TC (4)

      for(j=0;j<TC;j++) {
          s+=radio_getsignal();
          usleep(1000);
      }

      sprintf(tt,"FM-%.2f [%d]  %d  ",(float)freq/1000, i, s);
      dbg("%s\n",tt);

      fm_slot[i].s=s;
      fm_slot[i].chidx=i;

  }

  memcpy(o_fm_slot,fm_slot,sizeof(FM_SLOT)*FM_SLOT_N);

  qsort(fm_slot,FM_SLOT_N,sizeof(FM_SLOT),qmp_strength);

#define CUT_N (int)(FM_SLOT_N*0.8)

  cutoff=fm_slot[CUT_N-1].s;

  dbg("cutoff:%d\n",cutoff);

  maxidx0=maxidx1=-1;
  cnt=0;

  if (stat(ffradio_freq, &st) && mkdir(ffradio_freq,0755))
    perror(ffradio_freq);

  cnt = get_max_rno();

  for(i=0;i<FM_SLOT_N;i++) {
    if (o_fm_slot[i].s > cutoff) {
      if (maxidx0<0 || o_fm_slot[i].s > o_fm_slot[maxidx0].s) {
        maxidx0=maxidx1=i;
        flat=1;
      }
      else
      if (flat && o_fm_slot[i].s == o_fm_slot[maxidx0].s)
        maxidx1=i;
      else
        flat=0;

    } else
    if (maxidx0>=0) {
      int j;
#if     1
      ++cnt;

      int avg=(CHIDX_KHZ(maxidx0) + CHIDX_KHZ(maxidx1))/2;

      // skip if is already defined
      for(j=0;j<chanlistN;j++) {
        if (iabs(chanlist[j].freq - avg) < 20)
          break;
      }

      if (j < chanlistN)
        continue;

      add_new_station(cnt,avg);

      add_item_to_list(chanlist[chanlistN-1].name,cnt,avg);
#endif
      maxidx0=maxidx1=-1;
    }
  }

  load_chanlist();
ret:
  gtk_widget_destroy(scan_window);
  scan_window=NULL;
}
