#include <stdio.h>
#include <unistd.h>
#include <gtk/gtk.h>
#include "fftv.h"
#include "tv-freq.h"
#include <sys/types.h>
#include <linux/videodev.h>
#include "tv-audio.h"

static GtkWidget *radio_window;
static GtkWidget *radio_ui_menubar;
static GtkWidget *button_freq, *button_name, *button_adj_dec, *button_adj_inc;
static GtkWidget *image_station_prev, *image_station_next;
static GtkWidget *event_box_station_prev, *event_box_station_next;
static GtkWidget *button_name_label, *button_freq_label;
static GtkWidget *hscale;
static GtkWidget *vbox_top, *hbox_volume,*hbox_station, *button_speaker;
static GtkWidget *vbox_station_left, *vbox_station_right, *hbox_freq_adj;
static GtkWidget *image_speaker;
static GtkWidget *hseparator1,*vseparator;

static GtkWidget *mts_label;
static GtkWidget *vbox_mts, *hseparator_mts;
static GtkWidget *arrow_mts;

static GtkWidget *hseparator2;
static GtkWidget *hbox_record;
static GtkWidget *image_record, *image_pause, *image_stop;
static GtkWidget *event_box_record, *event_box_pause, *event_box_stop;


static void callback_button_station_prev( GtkWidget *widget,
                      gpointer   data )
{
    prev_channel();
}

static void callback_button_station_next( GtkWidget *widget,
                      gpointer   data )
{
    next_channel();
}


static void callback_button_enter_station_prev( GtkWidget *widget,
                      gpointer   data )
{
    gtk_image_set_from_file(GTK_IMAGE(image_station_prev),ICON_BASENAME "/down40-bd.png");
}

static void callback_button_enter_station_next( GtkWidget *widget,
                      gpointer   data )
{
    gtk_image_set_from_file(GTK_IMAGE(image_station_next),ICON_BASENAME "/up40-bd.png");
}


static void callback_button_leave_station_prev( GtkWidget *widget,
                      gpointer   data )
{
    gtk_image_set_from_file(GTK_IMAGE(image_station_prev),ICON_BASENAME "/down40.png");
}

static void callback_button_leave_station_next( GtkWidget *widget,
                      gpointer   data )
{
    gtk_image_set_from_file(GTK_IMAGE(image_station_next),ICON_BASENAME "/up40.png");
}


static gboolean delete_event( GtkWidget *widget,
                              GdkEvent  *event,
                              gpointer   data )
{
    do_exit_fftv();
    return FALSE;
}

static gboolean b_update_radio_hscale=FALSE;
void set_soundcard_volume(int vol);

static void callback_hscale(GtkRange *range, gpointer user_data)
{
  int val=(int)gtk_range_get_value(range);

  if (!b_update_radio_hscale)
    set_soundcard_volume(val);
}

static void callback_button_adj_dec(GtkButton *button, gpointer user_data)
{
  freq_fine_tune_down();
}

static void callback_button_adj_inc(GtkButton *button, gpointer user_data)
{
  freq_fine_tune_up();
}

void show_radio_list_window();
static void callback_button_freq(GtkButton *button, gpointer user_data)
{
  show_radio_list_window();
}

static void callback_button_name(GtkButton *button, gpointer user_data)
{
  show_radio_list_window();
}

void update_radio_hscale();
static void callback_button_speaker(GtkButton *button, gpointer user_data)
{
  soundcard_mute_toggle();
  update_radio_hscale();
}


static gboolean radio_key_eve(GtkWidget *widget, GdkEventKey *event, gpointer user_data)
{
  gchar chr=0;
  if (event->string)
    chr=event->string[0];
//    feed_xkey(event->keyval,event->state,chr);
//  printf("key %x %x %c\n",event->keyval,event->state,chr);
  feed_xkey(event->keyval,event->state,chr);
  return TRUE;
}

void draw_radio_freq(float freq)
{
  char tt[16];

  sprintf(tt,"%.2f MHz",freq);
  gtk_label_set_text(GTK_LABEL(button_freq_label),tt);
}

void draw_radio_station_name(char *name)
{
//  gtk_label_set_text(GTK_LABEL(button_name_label), name);

  if (!radio_window)
    return;

  gtk_label_set_text(GTK_LABEL(button_name_label),name);

  PangoFontDescription* font=pango_context_get_font_description
                             (gtk_widget_get_pango_context (button_name_label));

  pango_font_description_set_size(font,PANGO_SCALE * 20);
  gtk_widget_modify_font(button_name_label,font);

  GtkRequisition sz;
  gtk_widget_size_request(button_name_label, &sz);

  int scalex=2500 / (sz.width + 1) + 1;
  int scaley= 800 / (sz.height + 1) + 1;
  int scale=MIN(scalex,scaley);

  font=pango_context_get_font_description
             (gtk_widget_get_pango_context (button_name_label));

  pango_font_description_set_size(font,PANGO_SCALE*scale);
  gtk_widget_modify_font(button_name_label,font);


  gtk_window_set_title (GTK_WINDOW (radio_window), name);
}

int get_soundcard_volume();

void update_radio_hscale()
{
  int vol=get_soundcard_volume();

  dbg("radio vol:%d\n", vol);

  if (!radio_window)
    return;

  b_update_radio_hscale=TRUE;
  gtk_range_set_value(GTK_RANGE(hscale),vol);
  b_update_radio_hscale=FALSE;
}

extern int mts_mode_arrN, mts_mode_active_idx;
extern SOUND_MODE_M mts_mode_arr[];
void set_to_mts_mode_active_idx();
gint my_popup_handler (GtkWidget *widget, GdkEvent *event);
void mouse_scroll_callback(GtkWidget *widget,GdkEvent *event, gpointer data);

static void cb_update_menu_select(GtkWidget  *item,  gpointer data)
{
    /* Set the update policy for both scale widgets */
  int idx=GPOINTER_TO_INT(data);

  mts_mode_active_idx = idx;
  gtk_label_set_text(GTK_LABEL(mts_label),mts_mode_arr[idx].name);
  set_to_mts_mode_active_idx();
}


void set_radio_record_active_image()
{
    gtk_image_set_from_file(GTK_IMAGE(image_record),ICON_BASENAME "/rec-ui0.png");
}

static void callback_button_enter_record( GtkWidget *widget, gpointer   data)
{
  set_radio_record_active_image();
}

void set_radio_record_inactive_image()
{
    gtk_image_set_from_file(GTK_IMAGE(image_record),ICON_BASENAME "/rec-ui1.png");
}


static void callback_button_leave_record( GtkWidget *widget, gpointer   data)
{
    if (record_status!=RECORD_ON)
      set_radio_record_inactive_image();
}


void start_recording();
static void callback_button_start_recording( GtkWidget *widget, gpointer   data)
{
    start_recording();
}

void set_radio_pause_active_image()
{
    gtk_image_set_from_file(GTK_IMAGE(image_pause),ICON_BASENAME "/pause-ui0.png");
}

static void callback_button_enter_pause( GtkWidget *widget, gpointer   data)
{
  set_radio_pause_active_image();
}


void set_radio_pause_inactive_image()
{
    gtk_image_set_from_file(GTK_IMAGE(image_pause),ICON_BASENAME "/pause-ui1.png");
}

static void callback_button_leave_pause(GtkWidget *widget, gpointer   data)
{
    if (record_status!=RECORD_PAUSE)
      set_radio_pause_inactive_image();
}

void pause_recording();
static void callback_button_pause_recording( GtkWidget *widget, gpointer   data )
{
    pause_recording();
    callback_button_leave_record(NULL,NULL);
}

static void callback_button_enter_stop( GtkWidget *widget, gpointer   data)
{
    gtk_image_set_from_file(GTK_IMAGE(image_stop),ICON_BASENAME "/stop-ui0.png");
}

static void callback_button_leave_stop( GtkWidget *widget, gpointer   data)
{
    gtk_image_set_from_file(GTK_IMAGE(image_stop),ICON_BASENAME "/stop-ui1.png");
}

void stop_recording();
static void callback_button_stop_recording( GtkWidget *widget, gpointer data)
{
    stop_recording();
    callback_button_leave_record(NULL,NULL);
    callback_button_leave_pause(NULL,NULL);
}

void switch_to_tv();
void hide_radio_win();
void create_codec_window();
void create_about_window();


static GtkItemFactoryEntry radio_ui_menu_items[] = {
  { "/_Switch",         NULL,         NULL, 0, "<Branch>" },
  { "/_Switch/TV",   NULL,      switch_to_tv , 0, NULL },
  { "/_Switch/sep1",     NULL,         NULL, 0, "<Separator>" },
  { "/_Switch/Quit",     "<control>Q", do_exit_fftv, 0, NULL },
  { "/_Config",         NULL,         NULL, 0, "<Branch>" },
  { "/_Config/File name\\/codec", NULL, create_codec_window, 0, NULL },
  { "/_Help",         NULL,         NULL, 0, "<LastBranch>" },
  { "/_Help/About ffradio", NULL, create_about_window, 0, NULL },
};

static void get_radio_ui_main_menu( GtkWidget  *window)
{
  GtkItemFactory *item_factory;
  GtkAccelGroup *accel_group;
  gint nmenu_items = sizeof (radio_ui_menu_items) / sizeof (radio_ui_menu_items[0]);

  accel_group = gtk_accel_group_new ();

  item_factory = gtk_item_factory_new (GTK_TYPE_MENU_BAR, "<main>",
				       accel_group);

  gtk_item_factory_create_items (item_factory, nmenu_items, radio_ui_menu_items, NULL);

  gtk_window_add_accel_group (GTK_WINDOW (window), accel_group);

  radio_ui_menubar = gtk_item_factory_get_widget (item_factory, "<main>");
}



void create_radio_win()
{
    GdkColor prog_color;
    PangoFontDescription* font;


    /* Create a new radio_window */
    radio_window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
    gtk_window_set_wmclass(GTK_WINDOW(radio_window),"ffradio","ffradio");
    GtkWidget *win_icon=gtk_image_new_from_file(ICON_BASENAME "/fftv.png");
    gtk_window_set_icon(GTK_WINDOW(radio_window),
                  gtk_image_get_pixbuf(GTK_IMAGE(win_icon)));

    /* Set the radio_window title */
    gtk_window_set_title (GTK_WINDOW (radio_window), "ffradio");

    /* Set a handler for delete_event that immediately
     * exits GTK. */
    g_signal_connect (G_OBJECT (radio_window), "delete_event",
                      G_CALLBACK (delete_event), NULL);

    g_signal_connect(G_OBJECT(radio_window),"key-press-event", G_CALLBACK(radio_key_eve), NULL);

    gtk_container_set_border_width (GTK_CONTAINER (radio_window), 6);

    vbox_top = gtk_vbox_new (FALSE, 0);

    /* Put the table in the main radio_window */
    gtk_container_add (GTK_CONTAINER (radio_window), vbox_top);

    get_radio_ui_main_menu(radio_window);

    gtk_box_pack_start (GTK_BOX (vbox_top), radio_ui_menubar, FALSE, FALSE, 0);

    GtkWidget *hseparator0=gtk_hseparator_new();

    gtk_box_pack_start (GTK_BOX (vbox_top), hseparator0, FALSE, FALSE, 5);

    /* Create first button */
    button_freq = gtk_button_new();
    button_freq_label = gtk_label_new("100.1 Mhz");
    font=pango_context_get_font_description
                               (gtk_widget_get_pango_context (button_freq_label));
    pango_font_description_set_size(font,PANGO_SCALE*20);

    prog_color.pixel=prog_color.red=prog_color.green=prog_color.blue=0;
    prog_color.blue=65535;

    gtk_widget_modify_font(button_freq_label,font);
    gtk_widget_modify_fg(button_freq_label,GTK_STATE_NORMAL,&prog_color);
    gtk_widget_modify_fg(button_freq_label,GTK_STATE_PRELIGHT,&prog_color);
    gtk_container_add (GTK_CONTAINER (button_freq), button_freq_label);

    g_signal_connect (G_OBJECT (button_freq),"clicked",
      G_CALLBACK(callback_button_freq),NULL);


    button_name = gtk_button_new();
    button_name_label = gtk_label_new("");
    font=pango_context_get_font_description
                               (gtk_widget_get_pango_context (button_name_label));
    pango_font_description_set_size(font,PANGO_SCALE*25);

    prog_color.pixel=0;
    prog_color.red=255<<8;
    prog_color.green=255<<8;
    prog_color.blue=0<<8;

    gtk_widget_modify_font(button_name_label,font);
//   gtk_widget_modify_fg(button_name_label,GTK_STATE_NORMAL,&prog_color);
//   gtk_widget_modify_fg(button_name_label,GTK_STATE_PRELIGHT,&prog_color);
    gtk_container_add (GTK_CONTAINER (button_name), button_name_label);

    g_signal_connect (G_OBJECT (button_name),"clicked",
      G_CALLBACK(callback_button_name),NULL);


    vbox_station_left = gtk_vbox_new (TRUE, 0);
    gtk_box_pack_start (GTK_BOX (vbox_station_left), button_freq, TRUE, TRUE, 2);
    gtk_box_pack_start (GTK_BOX (vbox_station_left), button_name, TRUE, TRUE, 2);


    hscale = gtk_hscale_new_with_range (0,100,2);
    g_signal_connect (G_OBJECT (hscale), "value-changed",
                      G_CALLBACK (callback_hscale), NULL);

    button_speaker = gtk_button_new();
    image_speaker = gtk_image_new_from_file(ICON_BASENAME "/gnome-mixer-small.png");
    gtk_container_add (GTK_CONTAINER (button_speaker), image_speaker);

    g_signal_connect (G_OBJECT (button_speaker),"clicked",
      G_CALLBACK(callback_button_speaker),NULL);

    vbox_mts = gtk_vbox_new (FALSE, 0);

    GtkWidget *menu = gtk_menu_new ();

    int i;

    for(i=0;i<2;i++) {
      GtkWidget *item = gtk_menu_item_new_with_label (mts_mode_arr[i].name);

      gtk_widget_show(item);

      g_signal_connect (G_OBJECT (item), "activate",
                        G_CALLBACK (cb_update_menu_select), GINT_TO_POINTER(i));

      gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
    }


    GtkWidget *mts_button = gtk_button_new();
    mts_label = gtk_label_new(mts_mode_arr[mts_mode_active_idx].name);
    gtk_box_pack_start(GTK_BOX (vbox_mts), mts_label, TRUE,TRUE,0);
    hseparator_mts = gtk_hseparator_new();
    gtk_box_pack_start(GTK_BOX (vbox_mts), hseparator_mts, TRUE,TRUE,0);
    g_signal_connect_swapped (GTK_OBJECT (mts_button), "button_press_event",
          G_CALLBACK (my_popup_handler), GTK_OBJECT (menu));

    arrow_mts = gtk_arrow_new (GTK_ARROW_DOWN, GTK_SHADOW_OUT);
    gtk_box_pack_start(GTK_BOX (vbox_mts),arrow_mts,TRUE,TRUE,0);

    gtk_container_add (GTK_CONTAINER (mts_button), vbox_mts);

    hbox_volume = gtk_hbox_new (FALSE, 0);
    gtk_box_pack_start (GTK_BOX (hbox_volume), button_speaker, FALSE, FALSE, 0);
    gtk_box_pack_start (GTK_BOX (hbox_volume), mts_button, FALSE, FALSE, 0);
    gtk_box_pack_start (GTK_BOX (hbox_volume), hscale, TRUE, TRUE, 0);


    button_adj_dec = gtk_button_new_with_label ("-");
    button_adj_inc = gtk_button_new_with_label ("+");
    hbox_freq_adj = gtk_hbox_new (TRUE, 0);
    gtk_box_pack_start(GTK_BOX (hbox_freq_adj),button_adj_dec,TRUE,TRUE,0);
    gtk_box_pack_start(GTK_BOX (hbox_freq_adj),button_adj_inc,TRUE,TRUE,0);
    g_signal_connect (G_OBJECT (button_adj_dec),"clicked",
       G_CALLBACK(callback_button_adj_dec),NULL);
    g_signal_connect (G_OBJECT (button_adj_inc),"clicked",
       G_CALLBACK(callback_button_adj_inc),NULL);


    image_station_prev = gtk_image_new_from_file(ICON_BASENAME "/down40.png");
    image_station_next = gtk_image_new_from_file (ICON_BASENAME "/up40.png");
    event_box_station_prev = gtk_event_box_new();
    event_box_station_next = gtk_event_box_new();

    gtk_container_add (GTK_CONTAINER (event_box_station_prev), image_station_prev);
    gtk_container_add (GTK_CONTAINER (event_box_station_next), image_station_next);


    vbox_station_right = gtk_vbox_new (FALSE, 0);
    gtk_box_pack_start (GTK_BOX(vbox_station_right),event_box_station_next, FALSE, FALSE, 0);
    gtk_box_pack_start (GTK_BOX(vbox_station_right),event_box_station_prev, FALSE, FALSE, 0);
    gtk_box_pack_start (GTK_BOX(vbox_station_right),hbox_freq_adj, TRUE, TRUE, 5);


    g_signal_connect (G_OBJECT (event_box_station_prev), "button_press_event",
                      G_CALLBACK (callback_button_station_prev), NULL);
    g_signal_connect (G_OBJECT (event_box_station_next), "button_press_event",
                      G_CALLBACK (callback_button_station_next), NULL);

    g_signal_connect (G_OBJECT (event_box_station_prev), "enter-notify-event",
                      G_CALLBACK (callback_button_enter_station_prev), NULL);
    g_signal_connect (G_OBJECT (event_box_station_next), "enter-notify-event",
                      G_CALLBACK (callback_button_enter_station_next), NULL);

    g_signal_connect (G_OBJECT (event_box_station_prev), "leave-notify-event",
                      G_CALLBACK (callback_button_leave_station_prev), NULL);
    g_signal_connect (G_OBJECT (event_box_station_next), "leave-notify-event",
                      G_CALLBACK (callback_button_leave_station_next), NULL);

    hbox_station = gtk_hbox_new (FALSE, 0);
    vseparator=gtk_vseparator_new();
    gtk_box_pack_start(GTK_BOX(hbox_station),vbox_station_left,TRUE,TRUE, 0);
    gtk_box_pack_start (GTK_BOX(hbox_station),vseparator, FALSE, FALSE, 5);
    gtk_box_pack_start(GTK_BOX(hbox_station),vbox_station_right,FALSE,FALSE, 0);

    hseparator1=gtk_hseparator_new();


    hseparator2=gtk_hseparator_new();
    hbox_record = gtk_hbox_new (TRUE, 0);

    image_record = gtk_image_new_from_file(ICON_BASENAME "/rec-ui1.png");
    image_pause = gtk_image_new_from_file (ICON_BASENAME "/pause-ui1.png");
    image_stop = gtk_image_new_from_file (ICON_BASENAME "/stop-ui1.png");
    event_box_record = gtk_event_box_new();
    event_box_pause = gtk_event_box_new();
    event_box_stop = gtk_event_box_new();

    g_signal_connect (G_OBJECT (event_box_record), "enter-notify-event",
                      G_CALLBACK (callback_button_enter_record), NULL);
    g_signal_connect (G_OBJECT (event_box_record), "leave-notify-event",
                      G_CALLBACK (callback_button_leave_record), NULL);
    g_signal_connect (G_OBJECT (event_box_record), "button_press_event",
                      G_CALLBACK (callback_button_start_recording), NULL);

    g_signal_connect (G_OBJECT (event_box_pause), "enter-notify-event",
                      G_CALLBACK (callback_button_enter_pause), NULL);
    g_signal_connect (G_OBJECT (event_box_pause), "leave-notify-event",
                      G_CALLBACK (callback_button_leave_pause), NULL);
    g_signal_connect (G_OBJECT (event_box_pause), "button_press_event",
                      G_CALLBACK (callback_button_pause_recording), NULL);

    g_signal_connect (G_OBJECT (event_box_stop), "enter-notify-event",
                      G_CALLBACK (callback_button_enter_stop), NULL);
    g_signal_connect (G_OBJECT (event_box_stop), "leave-notify-event",
                      G_CALLBACK (callback_button_leave_stop), NULL);
    g_signal_connect (G_OBJECT (event_box_stop), "button_press_event",
                      G_CALLBACK (callback_button_stop_recording), NULL);

    gtk_container_add (GTK_CONTAINER (event_box_record), image_record);
    gtk_container_add (GTK_CONTAINER (event_box_pause), image_pause);
    gtk_container_add (GTK_CONTAINER (event_box_stop), image_stop);

    gtk_box_pack_start(GTK_BOX(hbox_record),event_box_record,TRUE,TRUE, 0);
    gtk_box_pack_start (GTK_BOX(hbox_record),event_box_pause, FALSE, FALSE, 0);
    gtk_box_pack_start(GTK_BOX(hbox_record),event_box_stop,FALSE,FALSE, 0);


    gtk_box_pack_start (GTK_BOX (vbox_top), hbox_station, TRUE, TRUE, 0);
    gtk_box_pack_start (GTK_BOX (vbox_top), hseparator1, FALSE, FALSE, 5);
    gtk_box_pack_start (GTK_BOX (vbox_top), hbox_volume, FALSE, FALSE, 0);
    gtk_box_pack_start (GTK_BOX (vbox_top), hseparator2, FALSE, FALSE, 5);
    gtk_box_pack_start (GTK_BOX (vbox_top), hbox_record, FALSE, FALSE, 0);


    gtk_widget_show_all (radio_window);

    g_signal_connect (G_OBJECT(radio_window), "scroll-event",
                      G_CALLBACK (mouse_scroll_callback), NULL);

    dbg("radio win created ..\n");
}


void show_radio_win()
{
  if (!radio_window)
    create_radio_win();
  else
    gtk_widget_show_all(radio_window);

  update_radio_hscale();
}


void hide_radio_list_window();

void hide_radio_win()
{
  gtk_widget_hide(radio_window);

  hide_radio_list_window();
}
