 #include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <time.h>
#include <gtk/gtk.h>
#include <gdk/gdkx.h>

#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xresource.h>
#include <X11/Xproto.h>

#include "reminder.h"

#include <string.h>
#include <ctype.h>

#include "tv-freq.h"
#include "fftv.h"
#include "xkey.h"
#include "vidstr.h"
#include "tv-win.h"

extern Window tv_win;
extern GtkWidget *gtv_win;
extern int fheight,fwidth;
extern Display *dpy;
extern XFontStruct     *afont, *afont_small;
extern long white, grey, black, cyan, blue, brown, gold;
extern GC gc, gc_sfont, gc_rec;
extern long foreground, background;
extern int fwidth,fheight;
extern int sfwidth,sfheight;


// GdkWindow *gdk_tv_win;


VIDSTR vidstr[vidstrN];
VIDSTR *pch_str=&vidstr[0],*prec_str=&vidstr[1],*pvol_str=&vidstr[2],
       *ppicadj_str=&vidstr[3],*pprog_str=&vidstr[4],
       *prec_output_status=&vidstr[5], *pchan_monitor=&vidstr[6],
       *pcurrent_time=&vidstr[7];

long current_time();

void clear_vidstr()
{
  bzero(vidstr,sizeof(vidstr));
}


#define CH_X 60
#define CH_Y 30

#define SHADOW_L (3)

static void draw3dStr(VIDSTR *pvstr,char *s, int x, int y, long bg_color)
{
  int i,j;
  int len=strlen(s);

  if (!tv_win)
    return;
#if 0
  if (gtv_win)
    gdk_flush();
#endif
  pvstr->x=x;
  pvstr->y=y;
  pvstr->yl=fheight+SHADOW_L+1;
  pvstr->xl=XTextWidth(afont,s,len)+SHADOW_L+1;

  if (x==VID_X_CENTER) {
    pvstr->x=(win_xl - pvstr->xl)/2;
  }

  set_vidstr(pvstr, pvstr->xl, pvstr->yl);

  set_vwin();

  XSetForeground(dpy,gc,bg_color);
  XFillRectangle(dpy,tv_win,gc,pvstr->x,pvstr->y,pvstr->xl,pvstr->yl);

  XSetForeground(dpy,gc,black);

  XDrawRectangle(dpy,tv_win,gc,pvstr->x+1,pvstr->y+1,pvstr->xl-3,pvstr->yl-3);

  for(i=0;i<SHADOW_L;i++)
    for(j=0;j<SHADOW_L;j++)
      XDrawString(dpy,tv_win,gc,pvstr->x+i+2,pvstr->y+afont->ascent+j+2,s,len);

  XSetForeground(dpy,gc,white);
  XDrawString(dpy,tv_win,gc,pvstr->x+1,pvstr->y+afont->ascent+1,s,len);

//  dbg("draw3dStr:%s\n",s);
#if 0
  XFlush(dpy);
#endif
}

void draw_radio_freq(float freq);
void draw_radio_station_name(char *s);

static GtkWidget *ch_label, *current_time_label, *vol_label;

static void draw_channel_str(char *s)
{
  GtkRequisition sz;

  if (TV_WIN_DISP_MODE_GRAB)
      slow_tv_win_frame_rate();

  gtk_label_set_text(GTK_LABEL(ch_label),s);
  gtk_widget_size_request(pch_str->button,&sz);

  set_vidstr(pch_str, sz.width, sz.height);

  set_vwin();

  gtk_widget_show_all(pch_str->button);
}


void draw_current_time()
{
  GtkRequisition sz;
  char st[6];
  struct tm ltm;
  time_t ltime=time(NULL);
  localtime_r(&ltime,&ltm);
  sprintf(st,"%d:%02d",ltm.tm_hour,ltm.tm_min);

  gtk_label_set_text(GTK_LABEL(current_time_label), st);
  gtk_widget_size_request(pcurrent_time->button,&sz);

  set_vidstr(pcurrent_time, sz.width, sz.height);

  set_vwin();

  gtk_widget_show_all(pcurrent_time->button);
}



void draw_channel_name_chidx()
{
  int rno=chanlist[chan_idx].rno;
  char st[32];

//  dbg("draw_channel_name_chidx\n");
  if (fftv_mode & FFTV_MODE_RADIO) {
    draw_radio_freq((float)chanlist[chan_idx].freq/1000);
    draw_radio_station_name(chanlist[chan_idx].name);
  }
  else {
    sprintf(st,"%d",rno);
    draw_channel_str(st);
  }
}

static gboolean callback_wait_clear_channel_name(gpointer ptr)
{
  if (pch_str->clear_time > current_time())
    return FALSE;

  if (pch_str->xl) {
    clr_vidstr(pch_str);
    clr_vidstr(pcurrent_time);
    set_vwin();
  }

  clear_remkeys();

  gtk_widget_hide(pch_str->button);
  gtk_widget_hide(pcurrent_time->button);

  return FALSE;
}

void draw_channel_name(char *s, int clear_sec)
{
  draw_channel_str(s);

  if (clear_sec) {
    pch_str->clear_time = current_time() + clear_sec;
    gtk_timeout_add(clear_sec * 1000, callback_wait_clear_channel_name,NULL);
  }
}

void draw_rec_output_status(char *s)
{
  prec_output_status->str=s;
  prec_output_status->period_sec=2;
  prec_output_status->bg_color=color_pink;
  prec_output_status->x = 20;
  prec_output_status->y = win_yl - 100;

  draw_tv_message(prec_output_status);
}


#define CLEAR_PERIOD_PICT_SELECT (5)

gboolean callback_wait_clear_pict_select(void *ptr)
{
  if (ppicadj_str->clear_time && current_time()>=ppicadj_str->clear_time) {
    pic_adj_select_menu_off();
  }

  return FALSE;
}


static GtkWidget *pict_image, *pict_label, *pict_hbox;
static GtkWidget *fixed;

char *icon_basename=ICON_BASENAME;

void draw_pict_adj_status(char *imgfname, int val)
{
  char fname[128];
  char valstr[16];

  sprintf(fname,"%s/%s.png",icon_basename,imgfname);
  sprintf(valstr,"%d",val);

  gtk_image_set_from_file(GTK_IMAGE(pict_image),fname);
  gtk_label_set_text(GTK_LABEL(pict_label),valstr);

  GtkRequisition sz;
  gtk_widget_size_request(ppicadj_str->button, &sz);

  ppicadj_str->x = 0;
  ppicadj_str->y = win_yl - sz.height;

  set_vidstr(ppicadj_str, sz.width, sz.height);

  gtk_fixed_move (GTK_FIXED(fixed),ppicadj_str->button, 0, ppicadj_str->y);

  set_vwin();

  gtk_widget_show_all(ppicadj_str->button);

  if (TV_WIN_DISP_MODE_GRAB)
      slow_tv_win_frame_rate();

#if     1
  if (fftv_mode & FFTV_MODE_TV) {
    ppicadj_str->clear_time = current_time() + CLEAR_PERIOD_PICT_SELECT;
    gtk_timeout_add(CLEAR_PERIOD_PICT_SELECT*1000,callback_wait_clear_pict_select,
       ppicadj_str);
  }
#endif
}


void draw_rec_status()
{
  char *fname;
  GtkWidget *image=prec_str->button;

  if (record_status==RECORD_ON) {
    fname = "rec";
  }
  else
  if (record_status==RECORD_PAUSE) {
    fname = "pause";
  } else
    return;

  char tfname[256];

  sprintf(tfname,"%s/%s.png",icon_basename,fname);

  gtk_image_set_from_file(GTK_IMAGE(image),tfname);

  GtkRequisition sz;
  gtk_widget_size_request(image,&sz);

  set_vidstr(prec_str, sz.width, sz.height);

  set_vwin();

  gtk_widget_show_all(image);
}


void disp_rec_status()
{
  if (record_status==RECORD_OFF || record_status==RECORD_STOP) {
    gtk_widget_hide(prec_str->button);
    prec_str->xl=0;
    set_vwin();
    return;
  }

  dbg("disp_rec_status\n");
  draw_rec_status();
}


static gboolean callback_wait_clear_vol(gpointer ptr)
{
  if (pvol_str->clear_time > current_time())
    return FALSE;

  if (pvol_str->xl) {
    clr_vidstr(pvol_str);
    set_vwin();
  }

  gtk_widget_hide(pvol_str->button);

  return FALSE;
}

#define CLEAR_PERIOD_VOL (1)

void draw_vol_status(char *s, int clear)
{
  GtkRequisition sz;

  gtk_label_set_text(GTK_LABEL(vol_label),s);
  gtk_widget_size_request(pvol_str->button,&sz);

  pvol_str->x = 0;
  pvol_str->y = win_yl - sz.height;
  set_vidstr(pvol_str, sz.width, sz.height);

  gtk_fixed_move (GTK_FIXED(fixed),pvol_str->button, 0, pvol_str->y);

  GdkColor vol_color;

  if (strcmp(s,"Mute")) {
    vol_color.green=35535;
    vol_color.pixel=vol_color.blue=vol_color.red=0;
    if (TV_WIN_DISP_MODE_GRAB)
      slow_tv_win_frame_rate();
  } else {
    vol_color.red=65535;
    vol_color.pixel=vol_color.blue=vol_color.green=0;
  }

  gtk_widget_modify_fg(vol_label,GTK_STATE_NORMAL,&vol_color);


  set_vwin();

  gtk_widget_show_all(pvol_str->button);

#if     1
  if (clear) {
    pvol_str->clear_time = current_time() + CLEAR_PERIOD_VOL;
    gtk_timeout_add(CLEAR_PERIOD_VOL * 1000,callback_wait_clear_vol, pvol_str);
  }
#endif
}


void draw_vol_off()
{
  gtk_widget_hide(pvol_str->button);
  pvol_str->xl=0;
  set_vwin();
}


static gboolean callback_wait_clear_message(gpointer ptr)
{
  VIDSTR *pvidstr=(VIDSTR *)ptr;

  if (pvidstr->clear_time > current_time())
    return FALSE;

  if (pvidstr->xl) {
    clr_vidstr(pvidstr);
    set_vwin();
  }

  return FALSE;
}

void draw_tv_message(VIDSTR *pvidstr)
{
  int len;

  len=strlen(pvidstr->str);
  pvidstr->m_str = realloc(pvidstr->m_str,(len+1));
  strcpy(pvidstr->m_str, pvidstr->str);

  draw3dStr(pvidstr,pvidstr->str,pvidstr->x,pvidstr->y,pvidstr->bg_color);

  if (pvidstr->period_sec) {
    pvidstr->clear_time = current_time() + pvidstr->period_sec;
    gtk_timeout_add(pvidstr->period_sec*1000,
       callback_wait_clear_message,pvidstr);
  }
}

GtkWidget *prog_label;

static gboolean callback_clear_reminder_message(gpointer ptr)
{
  if (pprog_str->clear_time > current_time())
    return FALSE;

  if (pprog_str->xl) {
    clr_vidstr(pprog_str);
    gtk_widget_hide(pprog_str->button);
    set_vwin();
  }

  return FALSE;
}

void draw_reminder_message(char *str)
{
  gtk_label_set_text(GTK_LABEL(prog_label),str);

  GtkRequisition sz;
  gtk_widget_size_request(pprog_str->button,&sz);

  pprog_str->period_sec=20;
  set_vidstr_xy(pprog_str, sz.width, sz.height, (win_xl - sz.width) /2, 20);

  gtk_fixed_move(GTK_FIXED(fixed),pprog_str->button, pprog_str->x,pprog_str->y);

  set_vwin();

  gtk_widget_show_all(pprog_str->button);

  pprog_str->clear_time = current_time() + pprog_str->period_sec;
  gtk_timeout_add(pprog_str->period_sec*1000,callback_clear_reminder_message,
     pprog_str);
}


void create_button_widget()
{

  dbg("create_button_widget ....\n");

  fixed = gtk_fixed_new ();

  prec_str->button = gtk_image_new();
#if 1
  gtk_widget_set_double_buffered (fixed, FALSE);
  gtk_widget_set_app_paintable (fixed, TRUE);
#endif
  gtk_fixed_put (GTK_FIXED(fixed),prec_str->button, prec_str->x, prec_str->y);

  gtk_container_add (GTK_CONTAINER (gtv_win), fixed);

  ppicadj_str->button = gtk_button_new();
  pict_hbox = gtk_hbox_new (FALSE, 0);
  pict_label = gtk_label_new(NULL);
  pict_image = gtk_image_new();

  PangoFontDescription* font=pango_context_get_font_description
                             (gtk_widget_get_pango_context (pict_label));
  pango_font_description_set_size(font,PANGO_SCALE*40);
  pango_font_description_set_family(font,"utopia");
  gtk_widget_modify_font(pict_label,font);

  gtk_box_pack_start (GTK_BOX (pict_hbox), pict_image, FALSE, FALSE, 0);
  gtk_box_pack_start (GTK_BOX (pict_hbox), pict_label, FALSE, FALSE, 0);
  gtk_container_add (GTK_CONTAINER (ppicadj_str->button), pict_hbox);

  gtk_widget_show(pict_hbox);
  gtk_widget_show(pict_label);
  gtk_widget_show(pict_image);
  gtk_fixed_put (GTK_FIXED(fixed),ppicadj_str->button, 100,100);


  pprog_str->button = gtk_button_new();
  prog_label = gtk_label_new(NULL);
  font=pango_context_get_font_description
                             (gtk_widget_get_pango_context (prog_label));
  pango_font_description_set_family(font,"utopia");
  pango_font_description_set_size(font,PANGO_SCALE*30);

  GdkColor prog_color;

  prog_color.red=65535;
  prog_color.pixel=prog_color.green=prog_color.blue=0;

  gtk_widget_modify_font(prog_label,font);
  gtk_widget_show(prog_label);
  gtk_widget_modify_fg(prog_label,GTK_STATE_NORMAL,&prog_color);
  gtk_container_add (GTK_CONTAINER (pprog_str->button), prog_label);
  gtk_fixed_put (GTK_FIXED(fixed),pprog_str->button, 100,100);


  pch_str->x = CH_X; pch_str->y = CH_Y;
  pch_str->button = gtk_button_new();
  ch_label = gtk_label_new(NULL);
  gtk_widget_show(ch_label);

  font=pango_context_get_font_description
                             (gtk_widget_get_pango_context (ch_label));
//  pango_font_description_set_family(font,"Helvetica");
  pango_font_description_set_family(font,"utopia");
  pango_font_description_set_size(font, PANGO_SCALE*35);
  pango_font_description_set_weight(font, PANGO_WEIGHT_BOLD);
//  dbg("family:%s\n",pango_font_description_get_family(font));

  GdkColor ch_color;

  ch_color.blue=65535;
  ch_color.pixel=ch_color.green=ch_color.red=0;

  gtk_widget_modify_font(ch_label,font);
  gtk_widget_modify_fg(ch_label,GTK_STATE_NORMAL,&ch_color);
  gtk_container_add (GTK_CONTAINER (pch_str->button), ch_label);
  gtk_fixed_put (GTK_FIXED(fixed),pch_str->button, pch_str->x, pch_str->y);


  pcurrent_time->button = gtk_button_new();
  current_time_label = gtk_label_new(NULL);
  gtk_widget_show(current_time_label);

  font=pango_context_get_font_description
                             (gtk_widget_get_pango_context (current_time_label));
  pango_font_description_set_family(font,"utopia");
  pango_font_description_set_size(font,PANGO_SCALE*40);

  pcurrent_time->x=CH_X + 90;
  pcurrent_time->y=CH_Y;
  gtk_widget_modify_font(current_time_label,font);
  gtk_container_add (GTK_CONTAINER (pcurrent_time->button), current_time_label);
  gtk_fixed_put (GTK_FIXED(fixed),pcurrent_time->button, pcurrent_time->x, pcurrent_time->y);


  pvol_str->button = gtk_button_new();
  vol_label = gtk_label_new(NULL);
  gtk_widget_show(vol_label);

  font=pango_context_get_font_description
                             (gtk_widget_get_pango_context (vol_label));
  pango_font_description_set_family(font,"utopia");
  pango_font_description_set_weight(font, PANGO_WEIGHT_BOLD);
  pango_font_description_set_size(font,PANGO_SCALE*35);

  GdkColor vol_color;

  vol_color.green=35535;
  vol_color.pixel=vol_color.blue=vol_color.red=0;

  pvol_str->x=0;
  pvol_str->y=0;
  gtk_widget_modify_font(vol_label,font);
  gtk_widget_modify_fg(vol_label,GTK_STATE_NORMAL,&vol_color);
  gtk_container_add (GTK_CONTAINER (pvol_str->button), vol_label);
  gtk_fixed_put (GTK_FIXED(fixed),pvol_str->button, pvol_str->x, pvol_str->y);


  if (record_status==RECORD_ON || record_status==RECORD_PAUSE)
    draw_rec_status();

  gtk_widget_show(fixed);
}
