#include <string.h>
#include <ctype.h>
#include <stdlib.h>
#include <stdarg.h>
#include <stdio.h>
#include <gtk/gtk.h>
#include "fftv.h"

void feed_digit_int();
void prev_channel();
void next_channel();
void soundcard_raise_vol();
void soundcard_lower_vol();
void return_last_channel();
void soundcard_mute_toggle();
void start_recording();
void pause_recording();
void preview_page_up();
void preview_page_down();
void preview_cursor_left();
void preview_cursor_right();
void preview_cursor_down();
void preview_cursor_up();
void preview_select_channel();
void fullscreen_toggle();
void fullscreen_toggle640();
void switch_sound_mode();
void stop_recording();
void freq_fine_tune_down();
void freq_fine_tune_up();
void tv_fm_switch();
void set_channel_by_name(char *chname);
void disp_channel_name_idx();
void switch_to_tv();
void switch_to_radio();
void set_lirc_cmd_state(char *state_name);
void pic_contrast_up();
void pic_contrast_down();
void pic_adj_select_menu();
void pic_adj_select_menu_off();
void pic_adj_select_up();
void pic_adj_select_down();
void pic_val_inc();
void pic_val_dec();
void preview_channel_stack();
void add_del_channel();
void disp_channel_name_idx_time();
void do_exit_fftv();
void PreviewSelMouseXY();
void scan_channel();
void swap_channel_monitor();
void toggle_channel_monitor();
void show_tv_list_window();
void hide_tv_list_window();
void show_tv_ui_list_window();
void hide_tv_ui_list_window();
void show_tv_ui_window();
void show_tv_list_window();
void set_channel_by_rno(int rno);
void center_mouse_pointer();
void show_current_tv_schedule();
void tv_video_shrink_left(), tv_video_shrink_right();
void tv_video_grow_left(), tv_video_grow_right();
void tv_video_shrink_bottom(), tv_video_shrink_top();
void tv_video_grow_bottom(), tv_video_grow_top();
void create_prog_list_window_view();

/*
 field 1 :
 field 2 :
 field 3 : arg type
           0 : no arg
           1 : char     (integer)
           2 : string
           3 : string string
           7 : string string string
           8 : int int
*/


typedef struct {
  char *funname;
  void (*action)();
  char argtype;
} FUNC_MAP;

void create_prog_list_window_current_channel();


FUNC_MAP func[]= {
  {"FeedDigit",feed_digit_int,1},
  {"NextChannel",next_channel,0},
  {"PrevChannel",prev_channel,0},
  {"RaiseVolume",soundcard_raise_vol,0},
  {"LowerVolume",soundcard_lower_vol,0},
  {"LastChannel",return_last_channel,0},
  {"Mute",soundcard_mute_toggle,0},
  {"StartRecord",start_recording,0},
  {"PauseRecord",pause_recording,0},
  {"StopRecord",stop_recording,0},
  {"PreviewPageUp",preview_page_up,0},
  {"PreviewPageDown",preview_page_down,0},
  {"PreviewCursorLeft",preview_cursor_left,0},
  {"PreviewCursorRight",preview_cursor_right,0},
  {"PreviewCursorDown",preview_cursor_down,0},
  {"PreviewCursorUp",preview_cursor_up,0},
  {"PreviewSelectChannel",preview_select_channel},
  {"FullScreenToggle",fullscreen_toggle},
  {"FullScreenToggle640",fullscreen_toggle640},
  {"SwitchSoundMode",switch_sound_mode,0},
  {"FreqFineTuneDown",freq_fine_tune_down,0},
  {"FreqFineTuneUp",freq_fine_tune_up,0},
  {"TV_FM_Switch",tv_fm_switch,0},
  {"SetChannelByName",set_channel_by_name,2},
  {"SetChannelByNo",set_channel_by_rno,1},
  {"DisplayChannel",disp_channel_name_idx,0},
  {"DisplayChannelTime",disp_channel_name_idx_time,0},
  {"SwitchToTV",switch_to_tv,0},
  {"SwitchToRadio",switch_to_radio,0},
#if LIRC
  {"SetLRC_State",set_lirc_cmd_state,2},
#endif
  {"PicAdjSelectMenu",pic_adj_select_menu,0},
  {"PicAdjSelectMenuOff",pic_adj_select_menu_off,0},
  {"PicAdjSelectUp",pic_adj_select_up,0},
  {"PicAdjSelectDown",pic_adj_select_down,0},
  {"PicValInc",pic_val_inc,0},
  {"PicValDec",pic_val_dec,0},
  {"PreviewChannelSelStack",preview_channel_stack,0},
  {"PreviewSelMouseXY",PreviewSelMouseXY,0},
  {"DoExit",do_exit_fftv,0},
  {"ScanChannel",scan_channel,0},
  {"SwapChannelMonitor",swap_channel_monitor,0},
  {"ToggleChannelMonitor",toggle_channel_monitor,0},
  {"ShowTV_UI_ListWindow",show_tv_ui_list_window,0},
  {"ShowTV_ListWindow", show_tv_list_window,0},
  {"ShowTV_UI_Window",show_tv_ui_window,0},
  {"HideTVListWindow",hide_tv_list_window,0},
  {"CenterMousePointer",center_mouse_pointer,0},
  {"ShowTV_Schedule", show_current_tv_schedule,0},
  {"TV_ScreenShrinkLeft", tv_video_shrink_left,0},
  {"TV_ScreenShrinkRight", tv_video_shrink_right,0},
  {"TV_ScreenGrowLeft", tv_video_grow_left,0},
  {"TV_ScreenGrowRight", tv_video_grow_right,0},
  {"TV_ScreenShrinkBottom", tv_video_shrink_bottom,0},
  {"TV_ScreenShrinkTop", tv_video_shrink_top,0},
  {"TV_ScreenGrowBottom", tv_video_grow_bottom,0},
  {"TV_ScreenGrowTop", tv_video_grow_top,0},
  {"ShowReminderListCurrent", create_prog_list_window_current_channel,0},
  {"ShowReminderListView", create_prog_list_window_view,0},
};

int funcN=sizeof(func)/sizeof(func[0]);

int isvarchr(int ch);
int isdig(int ch);
void mkstr(char *aa,char *bb,char *tt);
char *parse_Cstr(char *ffname, int lno, char *ss, char **end);
char *skip_spc(char *s);
int ispc(char c);
char *varname(char *s,char *fn);
void lper(char *ffname, int lno, char *fmt,...);


void lper2(char *ffname, int lno, char *fmt,...)
{
  va_list args;

  va_start(args, fmt);
  fprintf(stderr,"'%s' line %d:", ffname, lno);
  vfprintf(stderr, fmt, args);
  va_end(args);
  fprintf(stderr,"\n");
}


u_char takearg(char *ffname, int lno, char *ss, char **ostr, char **next)
{
  static char ooo[64];
  char *tt;
  ss=skip_spc(ss);
  *next=tt=ss;
  if (!*ss) return 0;
  if (isdig(*tt)) {
    while (isdig(*tt)) tt++;
    mkstr(ss,*next=tt,*ostr=ooo);
    return 1;
  }
  else
  if (*tt=='"') {
    *ostr=parse_Cstr(ffname, lno, tt,next);
    return 2;
  }
  return 0;
}

char *getstr(char *s, char **t)
{
  static char tt[80];
  char *u;
  while (ispc(*s) || *s==',') s++;
  u=s;
  while (isvarchr(*u) || (u>s && isdigit(*u))) u++;
  mkstr(s,u,tt);
  *t=u;
  return tt;
}

int parse_stmt(char *ffname, int lno,  char *ss,char **cmd, u_char *count)
{
  char fn[32], *arg,*tt;
  char tbuf[1024];
  u_short funidx[64],fuid;
  int typ,cmlen=0,fncnt=0,argtyp,ii;
  while (*ss) {
    if (!(ss=varname(ss,fn))) lper(ffname,lno,"func name expected %s",ss);
    for(fuid=0;fuid<funcN;fuid++)
      if (!strcmp(fn,func[fuid].funname))
        break;
    if (fuid==funcN) {
      lper2(ffname,lno,"func %s is not defined",fn);
      return 0;
    }
    ss=skip_spc(ss);
    if (*ss!='(') lper(ffname,lno,"( expected");
    ss++;
    funidx[fncnt++]=cmlen;
    argtyp=func[fuid].argtype;
    tbuf[cmlen++]=fuid;
    switch (argtyp) {
    case 0:
    case 4:
    case 5:
    case 6:
      typ=takearg(ffname, lno, ss,&arg,&tt);
      if (typ!=0)
        lper(ffname,lno,"No arg is expected");
      break;
    case 1:
      typ=takearg(ffname, lno, ss,&arg,&tt);
      if (typ!=1)
        lper(ffname, lno, ffname,lno,"integer is expected");
      tbuf[cmlen++]=atoi(arg);
      break;
    case 2:
      typ=takearg(ffname, lno, ss,&arg,&tt);
      if (typ!=2)
        lper(ffname,lno,"String is expected");
      strcpy(&tbuf[cmlen],arg);
      cmlen+=strlen(arg)+1;
      break;
    case 3:  /* (string,string) */
      typ=takearg(ffname, lno,ss,&arg,&tt);
      if (typ!=2)
        lper(ffname,lno,"String is expected");
      strcpy(&tbuf[cmlen],arg);
      cmlen+=strlen(arg)+1;
      tt=skip_spc(tt);
      if (*tt!=',') lper(ffname,lno,", is expected");
      tt++;
      typ=takearg(ffname, lno,tt,&arg,&tt);
      if (typ!=2)
        lper(ffname,lno,"String is expected");
      strcpy(&tbuf[cmlen],arg);
      cmlen+=strlen(arg)+1;
      break;
    case 7:  /* (string,string,string) */
      typ=takearg(ffname, lno,ss,&arg,&tt);
      if (typ!=2)
        lper(ffname,lno,"String is expected");
      strcpy(&tbuf[cmlen],arg);
      cmlen+=strlen(arg)+1;
      tt=skip_spc(tt);
      if (*tt!=',') lper(ffname,lno,", is expected");
      tt++;
      typ=takearg(ffname, lno,tt,&arg,&tt);
      if (typ!=2)
        lper(ffname,lno,"String is expected");
      strcpy(&tbuf[cmlen],arg);
      cmlen+=strlen(arg)+1;
      tt=skip_spc(tt);
      if (*tt!=',') lper(ffname,lno,", is expected");
      tt++;
      typ=takearg(ffname, lno,tt,&arg,&tt);
      if (typ!=2)
        lper(ffname,lno,"String is expected");
      strcpy(&tbuf[cmlen],arg);
      cmlen+=strlen(arg)+1;
      break;
    case 8: /* int int */
      typ=takearg(ffname, lno, ss,&arg,&tt);
      if (typ!=1)
        lper(ffname, lno, ffname,lno,"integer is expected");
      ii=atoi(arg);
      memcpy(&tbuf[cmlen],&ii,sizeof(int));
                  cmlen+=sizeof(int);
                  tt=skip_spc(tt);
                  if (*tt!=',') lper(ffname,lno,", is expected");
                  tt++;
                  ss=skip_spc(tt);
                  typ=takearg(ffname, lno, ss,&arg,&tt);
                  if (typ!=1)
                          lper(ffname, lno, ffname,lno,"integer is expected");
      ii=atoi(arg);
      memcpy(&tbuf[cmlen],&ii,sizeof(int));
                  cmlen+=sizeof(int);
          }

          ss=skip_spc(tt);
          if (*ss!=')')
                  lper(ffname,lno," ) expected");
          ss++;
          ss=skip_spc(ss);
          if (*ss==';') ss++;
  }

  if ((*cmd=(char *)malloc(cmlen))==NULL)
          lper(ffname,lno,"malloc err");
  *count=fncnt;
  memcpy(*cmd,tbuf,cmlen);
  return 1;
}

int ExecCommand(char *cmd, int count)
{
  int i;
  char *ttt,*uuu;
  char *pcmd;

  pcmd=cmd;
  for(i=0;i<count;i++) {
    u_char funo=*(cmd++);
    switch (func[funo].argtype) {
    case 0:
      func[funo].action();
      break;
    case 1:
      func[funo].action(*(cmd++));
      break;
    case 2:
      func[funo].action(cmd);
      cmd+=strlen(cmd)+1;
      break;
    case 3:
      func[funo].action(cmd,ttt=cmd+strlen(cmd)+1);
      cmd+=strlen(cmd)+1+strlen(ttt)+1;
      break;
    case 5:
      break;
    case 6:
      break;
    case 7:
      ttt=cmd+strlen(cmd)+1;
      uuu=ttt+strlen(ttt)+1;
      func[funo].action(cmd,ttt,uuu);
      cmd+=strlen(cmd)+1+strlen(ttt)+1+strlen(uuu)+1;
      break;
    case 8:
      break;
    }
  }

  return 1;
}

