#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "gtk/gtk.h"
#include "tv-freq.h"
#include "fftv.h"

static GtkWidget *tv_list_window = NULL;
static GtkWidget *vbox;
static GtkWidget *sw;
static GtkWidget *treeview;
static GtkWidget *menubar;

typedef struct
{
  gint  rno;
  gchar *name;
  gboolean enabled;
  gboolean editable;
  gchar *tv_schedule_html;
} Item;

enum
{
  COLUMN_NAME,
  COLUMN_RNO,
  COLUMN_ENABLED,
  COLUMN_EDITABLE,
  COLUMN_TV_SCHEDULE,
  NUM_COLUMNS
};

static GArray *channelArr = NULL;

static int qcmp_number(const void *aa, const void *bb)
{
  Item *a=(Item *)aa, *b=(Item *)bb;

  return b->rno - a->rno;
}

static void add_item(char *name, int rno, gboolean enabled, gchar *tv_schedule_html)
{
  Item foo;

  foo.name = g_strdup(name);
  foo.rno = rno;
  foo.enabled = enabled;
  foo.editable = TRUE;
  foo.tv_schedule_html = tv_schedule_html;

  g_array_append_vals (channelArr, &foo, 1);
}

static void tv_add_items ()
{

  g_return_if_fail (channelArr != NULL);

  int i;

  for(i=0;i<chanlistN;i++) {
    CHANLIST *pcha=&chanlist[i];

    add_item(pcha->name, pcha->rno, !(pcha->flag & CHAN_IGNORE), pcha->tv_schedule_html);
  }

  g_array_sort (channelArr,qcmp_number);
}



static GtkTreeModel *tv_create_model ()
{
  gint i = 0;
  GtkListStore *model;
  GtkTreeIter iter;

  /* create array */
  channelArr = g_array_sized_new (FALSE, FALSE, sizeof (Item), 1);

  tv_add_items ();

  /* create list store */
  model = gtk_list_store_new (NUM_COLUMNS,G_TYPE_STRING, G_TYPE_INT,
                              G_TYPE_BOOLEAN,G_TYPE_BOOLEAN, G_TYPE_STRING);

  /* add items */
  for (i = 0; i < channelArr->len; i++) {
      gtk_list_store_append (model, &iter);

      gtk_list_store_set (model, &iter,
			  COLUMN_NAME,
			  g_array_index (channelArr, Item, i).name,
			  COLUMN_RNO,
			  g_array_index (channelArr, Item, i).rno,
			  COLUMN_ENABLED,
			  g_array_index (channelArr, Item, i).enabled,
			  COLUMN_EDITABLE,
			  g_array_index (channelArr, Item, i).editable,
			  COLUMN_TV_SCHEDULE,
			  g_array_index (channelArr, Item, i).tv_schedule_html,
			  -1);
  }

  return GTK_TREE_MODEL (model);
}

int rename_station(int rno, const gchar *new);

static void
cell_edited (GtkCellRendererText *cell,
             const gchar         *path_string,
             const gchar         *new_text,
             gpointer             data)
{
  GtkTreeModel *model = (GtkTreeModel *)data;
  GtkTreePath *path = gtk_tree_path_new_from_string (path_string);
  GtkTreeIter iter;
  gint *column  = g_object_get_data (G_OBJECT (cell), "column");
  gchar *old_text = NULL;

  gtk_tree_model_get_iter (model, &iter, path);
  gint i = gtk_tree_path_get_indices (path)[0];

  gtk_tree_model_get (model, &iter, column, &old_text, -1);

  int rno = g_array_index (channelArr, Item, i).rno;

  switch (GPOINTER_TO_INT (column)) {
    case COLUMN_NAME:
      {
        if (rename_station(rno, new_text))
          break;

        g_free (old_text);
  for (i = 0; i < channelArr->len; i++) {
    g_free(g_array_index(channelArr, Item, i).name);
  }

        g_array_index (channelArr, Item, i).name = g_strdup (new_text);

        gtk_list_store_set (GTK_LIST_STORE (model), &iter, column,
                            g_array_index (channelArr, Item, i).name, -1);
      }
      break;
    case COLUMN_RNO:
      break;
    case COLUMN_ENABLED:
      break;
    case COLUMN_TV_SCHEDULE:
      {
        g_free (old_text);

        CHANLIST *pcha = rno2pcha(rno);
        free(pcha->tv_schedule_html);

        pcha->tv_schedule_html = g_strdup (new_text);
        g_array_index (channelArr, Item, i).tv_schedule_html = g_strdup (new_text);

        gtk_list_store_set (GTK_LIST_STORE (model), &iter, column,
                            g_array_index (channelArr, Item, i).tv_schedule_html, -1);
        save_channel(pcha);
      }
  }

  gtk_tree_path_free (path);
}

void delete_current_channel();
void add_current_channel();
int setChannelFreq_0(char *chname);

static gboolean toggled (GtkCellRendererToggle *cell, gchar *path_string, gpointer data)
{
  GtkTreeModel *model = GTK_TREE_MODEL (data);
  GtkTreeIter iter;
  GtkTreePath *path = gtk_tree_path_new_from_string (path_string);
  gboolean value;

  printf("toggled\n");

  gtk_tree_model_get_iter (model, &iter, path);
  gtk_tree_model_get (model, &iter, COLUMN_ENABLED, &value, -1);

  int i = gtk_tree_path_get_indices (path)[0];
  int rno=g_array_index (channelArr, Item, i).rno;
  char *chname=g_array_index (channelArr, Item, i).name;
  char tt[16];

  sprintf(tt,"%d",rno);

  setChannelFreq_0(tt);

  value = !value;

  if (value) {
    dbg("add ....%s\n",chname);
    add_current_channel();
  } else {
    dbg("delete ....%s\n",chname);
    delete_current_channel();
  }

  gtk_list_store_set (GTK_LIST_STORE (model), &iter, COLUMN_ENABLED, value, -1);

  gtk_tree_path_free (path);

  return TRUE;
}


static void tv_add_columns (GtkTreeView *treeview)
{
  GtkCellRenderer *renderer;
  GtkTreeModel *model = gtk_tree_view_get_model (treeview);


  /* number column */
  renderer = gtk_cell_renderer_text_new ();
  g_signal_connect (G_OBJECT (renderer), "edited",
                    G_CALLBACK (cell_edited), model);
  g_object_set_data (G_OBJECT (renderer), "column", (gint *)COLUMN_RNO);

  gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (treeview),
                                               -1, "Ch #", renderer,
                                               "text", COLUMN_RNO,
                                               NULL);
  /* name column */
  renderer = gtk_cell_renderer_text_new ();
  g_signal_connect (G_OBJECT (renderer), "edited",
                    G_CALLBACK (cell_edited), model);
  g_object_set_data (G_OBJECT (renderer), "column", (gint *)COLUMN_NAME);

  gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (treeview),
                                               -1, "Channel Name", renderer,
                                               "text", COLUMN_NAME,
                                               "editable", COLUMN_EDITABLE,
                                               NULL);

  // enabled column
  renderer = gtk_cell_renderer_toggle_new ();
  g_signal_connect (G_OBJECT (renderer), "toggled",
		    G_CALLBACK (toggled), model);

  g_object_set (G_OBJECT (renderer), "xalign", 0.0, NULL);

  gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (treeview),
					       -1, "Disp",
					       renderer,
					       "active", COLUMN_ENABLED,
					       NULL);


  /* program list column */
  renderer = gtk_cell_renderer_text_new ();
  g_signal_connect (G_OBJECT (renderer), "edited",
                    G_CALLBACK (cell_edited), model);
  g_object_set_data (G_OBJECT (renderer), "column", (gint *)COLUMN_TV_SCHEDULE);

  gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (treeview),
                                               -1, "TV Schedule html", renderer,
                                               "text", COLUMN_TV_SCHEDULE,
                                               "editable", COLUMN_EDITABLE,
                                               NULL);
}

static void callback_row_selected(GtkTreeSelection *treeselection,
                                  gpointer user_data)
{
  GtkTreeIter iter;
  GtkTreeModel *model = gtk_tree_view_get_model (GTK_TREE_VIEW(treeview));

  if (!gtk_tree_selection_get_selected (treeselection, NULL, &iter))
    return;

  gint i;
  GtkTreePath *path;

//  dbg("selected ...\n");

  path = gtk_tree_model_get_path (model, &iter);
  i = gtk_tree_path_get_indices (path)[0];
  gtk_tree_path_free(path);

  int rno=g_array_index (channelArr, Item, i).rno;

//  printf("selected %s\n",name);
  set_channel_by_rno(rno);
}

void hide_tv_list_window();

static gboolean callback_tv_list_win_delete()
{
  hide_tv_list_window();

  return TRUE;
}


void tv_set_selection_by_rno(int rno)
{

  if (!treeview)
    return;

  GtkTreeSelection *selection=gtk_tree_view_get_selection(GTK_TREE_VIEW(treeview));
  GtkTreeModel *model = gtk_tree_view_get_model(GTK_TREE_VIEW(treeview));
  GtkTreeIter iter;
  gboolean found=FALSE;

  if (!gtk_tree_model_get_iter_first(model, &iter))
    return;

//  dbg("tv_set_selection_by_rno %d\n", rno);
  do {
    int *trno;

    gtk_tree_model_get(model,&iter,COLUMN_RNO, &trno,-1);

    if (rno == trno) {
      found=TRUE;
      break;
    }
  } while (gtk_tree_model_iter_next(model, &iter));

  if (found)
    gtk_tree_selection_select_iter(selection,&iter);

  gtk_tree_view_scroll_to_cell(GTK_TREE_VIEW(treeview),
    gtk_tree_model_get_path(model,&iter),NULL,FALSE,0.0,0.0);
}


void hide_tv_list_window()
{
  if (tv_list_window)
    gtk_widget_hide(tv_list_window);
}


void destroy_tv_list_window()
{
  int i;

  for (i = 0; i < channelArr->len; i++) {
    g_free(g_array_index(channelArr, Item, i).name);
  }


  for (i = 0; i < channelArr->len; i++) {
    g_free(g_array_index(channelArr, Item, i).tv_schedule_html);
  }

  g_array_free(channelArr, TRUE);

  gtk_widget_destroy(tv_list_window);
  tv_list_window = NULL;
}


static void callback_rescan( GtkWidget *w, gpointer   data)
{
  scan_channel();
}


static GtkItemFactoryEntry menu_items[] = {
  { "/_Setup",         NULL,         NULL, 0, "<Branch>" },
  { "/_Setup/Rescan",   NULL,         callback_rescan, 0, NULL },
  { "/_Setup/sep1",     NULL,         NULL, 0, "<Separator>" },
  { "/_Setup/Close",     "<control>Q",hide_tv_list_window , 0, NULL },
};


static void get_main_menu( GtkWidget  *window)
{
  GtkItemFactory *item_factory;
  GtkAccelGroup *accel_group;
  gint nmenu_items = sizeof (menu_items) / sizeof (menu_items[0]);

  accel_group = gtk_accel_group_new ();

  item_factory = gtk_item_factory_new (GTK_TYPE_MENU_BAR, "<main>",
				       accel_group);

  gtk_item_factory_create_items (item_factory, nmenu_items, menu_items, NULL);

  gtk_window_add_accel_group (GTK_WINDOW (window), accel_group);

  menubar = gtk_item_factory_get_widget (item_factory, "<main>");
}
#if 0
static gboolean tv_list_key_eve(GtkWidget *widget, GdkEventKey *event, gpointer user_data)
{
  gchar chr=0;
  if (event->string)
    chr=event->string[0];
//    feed_xkey(event->keyval,event->state,chr);
//  printf("key %x %x %c\n",event->keyval,event->state,chr);
  feed_xkey(event->keyval,event->state,chr);
  return TRUE;
}
#endif


static void create_tv_list_window (void)
{

  /* create tv_list_window, etc */
  tv_list_window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_window_set_title (GTK_WINDOW (tv_list_window), "TV channel list");
  gtk_container_set_border_width (GTK_CONTAINER (tv_list_window), 5);

  g_signal_connect (G_OBJECT (tv_list_window), "delete_event",
                      G_CALLBACK (callback_tv_list_win_delete), NULL);

  vbox = gtk_vbox_new (FALSE, 5);

  GtkWidget *win_icon=gtk_image_new_from_file(ICON_BASENAME "/fftv.png");
  gtk_window_set_icon(GTK_WINDOW(tv_list_window),
     gtk_image_get_pixbuf(GTK_IMAGE(win_icon)));

  get_main_menu(tv_list_window);

  gtk_box_pack_start (GTK_BOX (vbox), menubar, FALSE, FALSE, 0);

  gtk_container_add (GTK_CONTAINER (tv_list_window), vbox);
#if 0
  gtk_box_pack_start (GTK_BOX (vbox),
                      gtk_label_new ("TV Channel List"),
                      FALSE, FALSE, 0);
#endif
  sw = gtk_scrolled_window_new (NULL, NULL);
  gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (sw),
                                       GTK_SHADOW_ETCHED_IN);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (sw),
                                  GTK_POLICY_AUTOMATIC,
                                  GTK_POLICY_AUTOMATIC);
  gtk_box_pack_start (GTK_BOX (vbox), sw, TRUE, TRUE, 0);

  /* create model */
  GtkTreeModel *model = tv_create_model ();

  /* create tree view */
  treeview = gtk_tree_view_new_with_model (model);
  g_object_unref (G_OBJECT (model));
  gtk_tree_view_set_rules_hint (GTK_TREE_VIEW (treeview), TRUE);

  PangoFontDescription *font=pango_context_get_font_description
                             (gtk_widget_get_pango_context (treeview));
  pango_font_description_set_size(font,PANGO_SCALE*13);
  gtk_widget_modify_font(treeview, font);

  GtkTreeSelection *tree_selection =
     gtk_tree_view_get_selection (GTK_TREE_VIEW (treeview));
  gtk_tree_selection_set_mode (tree_selection, GTK_SELECTION_SINGLE);

  tv_add_columns (GTK_TREE_VIEW (treeview));

  gtk_container_add (GTK_CONTAINER (sw), treeview);

  gtk_window_set_default_size (GTK_WINDOW (tv_list_window), 360, 303);


  g_signal_connect (G_OBJECT (tv_list_window), "delete_event",
                    G_CALLBACK (gtk_main_quit), NULL);

  gtk_widget_show_all (tv_list_window);

  g_signal_connect (G_OBJECT(tree_selection), "changed",
                    G_CALLBACK (callback_row_selected), NULL);
#if 0
  g_signal_connect(G_OBJECT(tv_list_window),"key-press-event",
                     G_CALLBACK(tv_list_key_eve), NULL);
#endif
}

void show_tv_list_window()
{
  if (tv_list_window) {
    gtk_window_present(GTK_WINDOW(tv_list_window));
  }
  else
    create_tv_list_window();

  if (pchan_current)
    tv_set_selection_by_rno(pchan_current->rno);
}


