#include <stdio.h>
#include <sys/types.h>
#include <linux/videodev.h>
#include <sys/ioctl.h>
#include <string.h>
#include <stdlib.h>
#include <gtk/gtk.h>
#include "fftv.h"

static int active_vid_src_index;

void setVideoSource(int fd, char *vidname)
{
    int i;
    struct video_capability vid_cap;

    if (-1 == ioctl(fd,VIDIOCGCAP,&vid_cap)) {
      fprintf(stderr,"ioctl(h->fd,VIDIOCGCAP,&capN failed");
    }

    for (i = 0; i < vid_cap.channels; i++) {
        struct video_channel vidcha;

        bzero(&vidcha,sizeof(vidcha));
	vidcha.channel = i;
        ioctl(fd,VIDIOCGCHAN,&vidcha);

        if (!strcmp(vidcha.name,vidname)) {
          vidcha.norm=1;
          ioctl(fd,VIDIOCSCHAN,&vidcha);
          active_vid_src_index = i;
          return;
        }
    }

    p_err("cannot match the video source name %s",vidname);
}


static GtkWidget *vid_src_window = NULL;

static int new_select_idx;

static gboolean close_application( GtkWidget *widget, GdkEvent  *event,
                                   gpointer   data )
{
  active_vid_src_index = (int) new_select_idx;

  struct video_channel vidcha;

  bzero(&vidcha,sizeof(vidcha));
  ioctl(vfd, VIDIOCGCHAN, &vidcha);
  vidcha.channel = active_vid_src_index;

  vidcha.norm=1;

  ioctl(vfd,VIDIOCSCHAN,&vidcha);

//  printf("selected %x\n", active_vid_src_index);

  gtk_widget_destroy(vid_src_window);
  vid_src_window = NULL;

  return TRUE;
}


static void callback_button_clicked( GtkWidget *widget, gpointer   data)
{
   new_select_idx = GPOINTER_TO_INT(data);
}

extern GtkWidget *tv_ui_window;

void create_vid_src_window()
{
    GtkWidget *box1;
    GtkWidget *box2;
    GtkWidget *button, *button_close;
    GtkWidget *separator;
    GtkWidget *frame;
    struct video_capability vid_cap;

    new_select_idx = active_vid_src_index;

    if (-1 == ioctl(vfd,VIDIOCGCAP,&vid_cap)) {
      fprintf(stderr,"ioctl(h->fd,VIDIOCGCAP,&capN failed");
    }

    vid_src_window = gtk_window_new (GTK_WINDOW_TOPLEVEL);

    gint uix, uiy;
    gtk_window_get_position(GTK_WINDOW(tv_ui_window), &uix, &uiy);
    gtk_window_move(GTK_WINDOW(vid_src_window), uix, uiy);

    g_signal_connect (G_OBJECT (vid_src_window), "delete_event",
                      G_CALLBACK (close_application),
                      NULL);

    gtk_window_set_title (GTK_WINDOW (vid_src_window), "TV system");
    gtk_container_set_border_width (GTK_CONTAINER (vid_src_window), 3);

    frame = gtk_frame_new("Select a video source");
    gtk_container_add (GTK_CONTAINER (vid_src_window), frame);

    box1 = gtk_vbox_new (FALSE, 0);
    gtk_container_add (GTK_CONTAINER (frame), box1);

    box2 = gtk_vbox_new (FALSE, 10);
    gtk_container_set_border_width (GTK_CONTAINER (box2), 10);
    gtk_box_pack_start (GTK_BOX (box1), box2, TRUE, TRUE, 0);

    int i;
    GSList *group=NULL;

    for(i=0;i<vid_cap.channels;i++) {
      struct video_channel vidcha;

      bzero(&vidcha,sizeof(vidcha));
      vidcha.channel = i;
      ioctl(vfd,VIDIOCGCHAN,&vidcha);

      button = gtk_radio_button_new_with_label (group, vidcha.name);
      gtk_box_pack_start (GTK_BOX (box2), button, TRUE, TRUE, 0);

      group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (button));

      g_signal_connect (G_OBJECT (button), "clicked",
         G_CALLBACK (callback_button_clicked), (gpointer) i);

      if (i==active_vid_src_index)
        gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button), TRUE);

    }

    separator = gtk_hseparator_new ();
    gtk_box_pack_start (GTK_BOX (box1), separator, FALSE, TRUE, 0);

    box2 = gtk_vbox_new (FALSE, 10);
    gtk_container_set_border_width (GTK_CONTAINER (box2), 10);
    gtk_box_pack_start (GTK_BOX (box1), box2, FALSE, TRUE, 0);

    button_close = gtk_button_new_with_label ("OK");

    g_signal_connect_swapped (G_OBJECT (button_close), "clicked",
                              G_CALLBACK (close_application),
                              G_OBJECT (vid_src_window));
    gtk_box_pack_start (GTK_BOX (box2), button_close, TRUE, TRUE, 0);
    GTK_WIDGET_SET_FLAGS (button_close, GTK_CAN_DEFAULT);
    gtk_widget_grab_default (button_close);

    gtk_widget_show_all (vid_src_window);

    return;
}
