#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>
#include <gdk/gdk.h>
#include <gdk/gdkkeysyms.h>
#include <X11/XF86keysym.h>
#include "xkey.h"
#include "fftv.h"

typedef struct {
  char *keyname;
  guint xkey;
} KEYMAP;


#define CTRL (0x100)
#define SHFT (0x200)
#define ALT (0x400)
#define WIN (0x800)


KEYMAP kmap[]={
{"space",GDK_space},{"!",GDK_exclam},{"\"",GDK_quotedbl},{"#",GDK_numbersign},
{"$",GDK_dollar},{"%",GDK_percent},{"&",GDK_ampersand},{"'",GDK_quoteright},
{"(",GDK_parenleft},{")",GDK_parenright},{"*",GDK_asterisk},{"+",GDK_plus},
{",",GDK_comma},{"minus",GDK_minus},{".",GDK_period},{"/",GDK_slash},
{"0",GDK_0},{"1",GDK_1},{"2",GDK_2},{"3",GDK_3},{"4",GDK_4},{"5",GDK_5},{"6",GDK_6},
{"7",GDK_7},{"8",GDK_8},{"9",GDK_9},
{":",GDK_colon},{";",GDK_semicolon},{"<",GDK_less},{"=",GDK_equal},{">",GDK_greater},
{"?",GDK_question},{"@",GDK_at},
{"A",GDK_A},{"B",GDK_B},{"C",GDK_C},{"D",GDK_D},{"E",GDK_E},{"F",GDK_F},{"G",GDK_G},
{"H",GDK_H},{"I",GDK_I},{"J",GDK_J},{"K",GDK_K},{"L",GDK_L},{"M",GDK_M},{"N",GDK_N},
{"O",GDK_O},{"P",GDK_P},{"Q",GDK_Q},{"R",GDK_R},{"S",GDK_S},{"T",GDK_T},{"U",GDK_U},
{"V",GDK_V},{"W",GDK_W},{"X",GDK_X},{"Y",GDK_Y},{"Z",GDK_Z},
{"brl",GDK_bracketleft},{"\\",GDK_backslash},{"brr",GDK_bracketright},
{"^",GDK_asciicircum},{"_",GDK_underscore},{"`",GDK_grave},
{"a",GDK_a},{"b",GDK_b},{"c",GDK_c},{"d",GDK_d},{"e",GDK_e},{"f",GDK_f},{"g",GDK_g},
{"h",GDK_h},{"i",GDK_i},{"j",GDK_j},{"k",GDK_k},{"l",GDK_l},{"m",GDK_m},{"n",GDK_n},
{"o",GDK_o},{"p",GDK_p},{"q",GDK_q},{"r",GDK_r},{"s",GDK_s},{"t",GDK_t},{"u",GDK_u},
{"v",GDK_v},{"w",GDK_w},{"x",GDK_x},{"y",GDK_y},{"z",GDK_z},
{"{",GDK_braceleft}, {"|",GDK_bar}, {"}",GDK_braceright},{"~",GDK_asciitilde},

{"Backspace",GDK_BackSpace},{"Enter",GDK_Return},{"Tab",GDK_Tab},

{"Delete",GDK_Delete},{"Home",GDK_Home},{"End",GDK_End},{"Insert",GDK_Insert},
{"PageUp",GDK_Prior},{"PageDown",GDK_Next},
{"Left",GDK_Left},{"Right",GDK_Right},{"Up",GDK_Up},{"Down",GDK_Down},

{"F1",GDK_F1},{"F2",GDK_F2},{"F3",GDK_F3},{"F4",GDK_F4},{"F5",GDK_F5},{"F6",GDK_F6},
{"F7",GDK_F7},{"F8",GDK_F8},{"F9",GDK_F9},{"F10",GDK_F10},
{"F11",GDK_F11},{"F12",GDK_F12},
{"Print",GDK_Print},
{"KP0",GDK_KP_0},{"KP1",GDK_KP_1},{"KP2",GDK_KP_2},{"KP3",GDK_KP_3},{"KP4",GDK_KP_4},
{"KP5",GDK_KP_5},{"KP6",GDK_KP_6},{"KP7",GDK_KP_7},{"KP8",GDK_KP_8},{"KP9",GDK_KP_9},
{"KP/",GDK_KP_Divide},{"KP*",GDK_KP_Multiply},{"KP-",GDK_KP_Subtract},
{"KP+",GDK_KP_Add},{"KPenter",GDK_KP_Enter},{"KP.",GDK_KP_Decimal},
{"Escape",GDK_Escape}, {"Pause",GDK_Pause},
{"MouseButton1",MOUSE_BUTTON1},
{"MouseButton2",MOUSE_BUTTON2},
{"MouseButton3",MOUSE_BUTTON3},
{"MouseButton4",MOUSE_BUTTON4},
{"MouseButton5",MOUSE_BUTTON5},
{"RaiseVolume",XF86XK_AudioRaiseVolume},
{"LowerVolume",XF86XK_AudioLowerVolume},
{"Mute",XF86XK_AudioMute},
{"Media",XF86XK_AudioMedia},
{"HomePage",XF86XK_HomePage},
};

int kmapN=sizeof(kmap)/sizeof(kmap[0]);

typedef struct {
  u_short xkey;
  char *cmds;
  char cmdsN;
} XK_FF;

typedef struct {
  char *cmd_state;
  XK_FF *xk_ff;
  int xk_ffN;
} XK_FF_STATE;

static XK_FF *xk_ff=NULL;
static int xk_ffN=0;
XK_FF_STATE xk_ff_state[8];
int xk_ff_stateN, xk_ff_state_idx=0;

void lper(char *ffname, int lno, char *fmt,...);
void lper2(char *ffname, int lno, char *fmt,...);

int set_xkey_cmd_state(char *state_name)
{
  int i;

  if (!xk_ff_stateN)
    return 0;

  dbg("set_xkey_cmd_state: %s\n", state_name);

  for(i=0;i<xk_ff_stateN;i++)
    if (!strcmp(xk_ff_state[i].cmd_state,state_name)) {
      xk_ff=xk_ff_state[i].xk_ff;
      xk_ffN=xk_ff_state[i].xk_ffN;
      xk_ff_state_idx=i;
      return 0;
    }
  return 0;
  p_err("state_name %s is not defined",state_name);
}

char *get_xkey_cmd_state()
{
   return xk_ff_state[xk_ff_state_idx].cmd_state;
}

static u_short parse_key(char *tedname, int lno, char *uu,char *vv)
{
  int i;
  char *ww,*xx,*yy;
  u_short kmod,kname=0;

  while (*uu && uu<vv) {
    if (*uu!='[') lper(tedname,lno,"cannot find the 1st [");
    xx=++uu;
    kmod=0;
    yy=xx;
    while (*yy && *yy!='-' && *yy!=']') yy++;
    while (*xx!='-' && *xx!=']' && xx<vv) {
      if (*yy=='-')
      switch (*xx) {
      case 'c':
        kmod|=CTRL;
        break;
      case 'a':
        kmod|=ALT;
        break;
      case 's':
        kmod|=SHFT;
        break;
      case 'w':
        kmod|=WIN;
        break;
      }
      xx++;
    }
    if (!kmod) {
      if (*xx==']') {
        ww=xx;
        *xx=0;
      }
      else lper(tedname,lno,"cannot find ]");
      xx=uu;
    }
    else
    if (*xx=='-') {
      ww=++xx;
      while (*ww!=']' && ww<vv) ww++;
      if (*ww!=']') lper(tedname,lno,"cannot find ]");
      else *ww=0;
    } else lper(tedname,lno,"error");

    for(i=0;i<kmapN;i++)
      if (!strcmp(xx,kmap[i].keyname)){
        kname=i;
        break;
      }

    if (i==kmapN) lper(tedname,lno,"unknown key %s",xx);

    kname|=kmod;
    break;
  }

  return kname;
}


extern char *skip_spc(char *s), *to_spc(char *s);
int parse_stmt(char *ffname, int lno,  char *ss,char **cmd, u_char *count);
extern char *fftv_cmd_state_str;

void init_xk_ff()
{
  FILE *fp;
  XK_FF_STATE *pxk_ff_state=NULL;
  int lineno=0;
  char infname[256];
  char *fftv_xkey;

  if (xk_ff) {
    free(xk_ff);
    xk_ff=NULL;
    xk_ffN=0;
  }

  if ((fftv_xkey=getenv("FFTV_XKEY")))
    strcpy(infname,fftv_xkey);
  else
    sprintf(infname,"%s/fftv_xkey.map",FFTV_X11);

//  dbg("fname:%s\n",infname);
  if ((fp=fopen(infname,"r"))==NULL) {
    return;
  }

  while (!feof(fp)) {
    char line[128],*ss;
    int slen;

    bzero(line,sizeof(line));
    fgets(line,sizeof(line),fp);
    lineno++;

    ss=skip_spc(line);
    slen=strlen(ss);

    if (*ss=='#' || slen < 2)
      continue;

    if (ss[slen -1]=='\n')
      ss[slen-1]=0;


    if (*ss=='!') {
      ss=skip_spc(ss+1);

      if (!(*ss))
        p_err("Expect state name after !  %s\n",line);

      pxk_ff_state=&xk_ff_state[xk_ff_stateN];
      bzero(pxk_ff_state,sizeof(XK_FF_STATE));

      pxk_ff_state->cmd_state=strdup(ss);
      xk_ff_stateN++;
    } else {
      char *tt=to_spc(ss);
      char *cmds,cmdsN;
      u_short xkey;

      xkey=parse_key(infname,lineno,ss,tt);

      pxk_ff_state->xk_ff=
        realloc(pxk_ff_state->xk_ff,sizeof(XK_FF)*(pxk_ff_state->xk_ffN+1));
      pxk_ff_state->xk_ff[pxk_ff_state->xk_ffN].xkey=xkey;

      ss=skip_spc(tt);
      parse_stmt(infname,lineno,ss,&cmds,&cmdsN);
      pxk_ff_state->xk_ff[pxk_ff_state->xk_ffN].cmds=cmds;
      pxk_ff_state->xk_ff[pxk_ff_state->xk_ffN].cmdsN=cmdsN;
      pxk_ff_state->xk_ffN++;
    }
  }


  if (fftv_cmd_state_str)
    set_xkey_cmd_state(fftv_cmd_state_str);
  else
    set_fftv_cmd_state("Normal");

  return;
}

static char shift_chr[]=
"ABCDEFGHIJKLMNOPQRSTUVWXYZ~!@#$%^&*()_+|{\"}:?><";

static u_short map_xkey(guint skey,int state,int chr)
{
  int km=0,i;

  if (skey>0x7f&&skey<=0xff)
    skey=chr;
  if (state&(GDK_CONTROL_MASK|GDK_MOD1_MASK))
    skey=tolower(skey);

  for(i=0;i<kmapN;i++) {
    if (kmap[i].xkey == skey)
      break;
  }
  if (i==kmapN)
    return 0xffff;
  km=i;

//  dbg("%x %d\n",skey,km);

  if (state&(GDK_MOD1_MASK|GDK_MOD5_MASK))
    km|=ALT;
  if (state&GDK_MOD4_MASK)
    km|=WIN;
  if (state&GDK_CONTROL_MASK)
    km|=CTRL;
  if ((state&GDK_SHIFT_MASK) && (skey>255 || (strchr(shift_chr,skey) && km&(ALT|CTRL))))
    km|=SHFT;
//  dbg("iii %x\n",km);
  return km;
}

int ExecCommand(char *cmd, int count);

void feed_xkey(int skey, int state, int chr)
{
  u_short xkey=map_xkey(skey,state,chr);
  int i;

  for(i=0;i<xk_ffN;i++) {
    if (xk_ff[i].xkey!=xkey)
      continue;

    ExecCommand(xk_ff[i].cmds,xk_ff[i].cmdsN);
    return;
  }

  return;
}
